@@ -2,7 +2,7 @@
 /*
  * ESDM entropy source manager and external interface
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  */
 
 #define pr_fmt(fmt) KBUILD_MODNAME ": " fmt
@@ -11,7 +11,11 @@
 #include <linux/debugfs.h>
 #include <linux/device.h>
 #include <linux/fs.h>
+#include <linux/kernel.h>
 #include <linux/module.h>
+#include <linux/notifier.h>
+#include <linux/suspend.h>
+#include <linux/reboot.h>
 #include <linux/version.h>
 
 #include "esdm_es_ioctl.h"
@@ -20,6 +24,7 @@
 #include "esdm_es_mgr_irq.h"
 #include "esdm_es_mgr_sched.h"
 #include "esdm_es_timer_common.h"
+#include "esdm_drbg_kcapi.h"
 #include "esdm_testing.h"
 
 /* Only panic the kernel on permanent health failure if this variable is true */
@@ -88,6 +93,7 @@ static long esdm_cdev_ioctl(struct file *filp, unsigned int cmd,
 	mutex_lock(&esdm_cdev_lock);
 
 	switch (cmd) {
+#ifdef ESDM_ES_IRQ
 	case ESDM_IRQ_AVAIL_ENTROPY:
 	case ESDM_IRQ_ENT_BUF_SIZE:
 	case ESDM_IRQ_ENT_BUF:
@@ -95,6 +101,8 @@ static long esdm_cdev_ioctl(struct file *filp, unsigned int cmd,
 	case ESDM_IRQ_STATUS:
 		ret = esdm_es_mgr_irq_ioctl(cmd, arg);
 		break;
+#endif /* ESDM_ES_IRQ */
+#ifdef ESDM_ES_SCHED
 	case ESDM_SCHED_AVAIL_ENTROPY:
 	case ESDM_SCHED_ENT_BUF_SIZE:
 	case ESDM_SCHED_ENT_BUF:
@@ -102,6 +110,7 @@ static long esdm_cdev_ioctl(struct file *filp, unsigned int cmd,
 	case ESDM_SCHED_STATUS:
 		ret = esdm_es_mgr_sched_ioctl(cmd, arg);
 		break;
+#endif /* ESDM_ES_SCHED */
 	default:
 		ret = -ENOIOCTLCMD;
 		break;
@@ -116,7 +125,9 @@ static const struct file_operations esdm_cdev_fops = {
 	.open = esdm_cdev_open,
 	.release = esdm_cdev_release,
 	.unlocked_ioctl = esdm_cdev_ioctl,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 12, 0)
 	.llseek = no_llseek,
+#endif
 };
 
 static int __init esdm_es_mgr_dev_init(void)
@@ -125,13 +136,18 @@ static int __init esdm_es_mgr_dev_init(void)
 	struct device *device;
 	int ret;
 
+	/* make creation of all devices atomic */
+	mutex_lock(&esdm_cdev_lock);
+
 	esdm_es_class = class_create(
 #if LINUX_VERSION_CODE < KERNEL_VERSION(6, 4, 0)
 		THIS_MODULE,
 #endif
 		KBUILD_MODNAME);
-	if (IS_ERR(esdm_es_class))
+	if (IS_ERR(esdm_es_class)) {
+		mutex_unlock(&esdm_cdev_lock);
 		return PTR_ERR(esdm_es_class);
+	}
 
 	if (esdm_major) {
 		dev = MKDEV(esdm_major, 0);
@@ -143,18 +159,27 @@ static int __init esdm_es_mgr_dev_init(void)
 		esdm_major = MAJOR(dev);
 	}
 
-	if (ret < 0)
+	if (ret < 0) {
+		pr_warn("ESDM cdev class allocation failed\n");
 		goto err;
+	}
 
 	cdev_init(&esdm_cdev, &esdm_cdev_fops);
-	cdev_add(&esdm_cdev, dev, ESDM_MAX_MINORS);
+	ret = cdev_add(&esdm_cdev, dev, ESDM_MAX_MINORS);
+	if (ret < 0) {
+		pr_warn("ESDM cdev creation failed\n");
+		goto err_cdev;
+	}
 
 	device = device_create(esdm_es_class, NULL, dev, NULL, KBUILD_MODNAME);
 	if (IS_ERR(device)) {
+		pr_warn("ESDM cdev /sys allocation failed\n");
 		ret = PTR_ERR(device);
 		goto err_cdev;
 	}
 
+	mutex_unlock(&esdm_cdev_lock);
+
 	pr_info("ESDM user space interface available (major number %u)\n",
 		esdm_major);
 
@@ -165,6 +190,7 @@ err_cdev:
 	unregister_chrdev_region(MKDEV(esdm_major, 0), ESDM_MAX_MINORS);
 err:
 	class_destroy(esdm_es_class);
+	mutex_unlock(&esdm_cdev_lock);
 	return ret;
 }
 
@@ -175,7 +201,6 @@ static void __exit esdm_es_mgr_dev_fini(void)
 	device_destroy(esdm_es_class, MKDEV(esdm_major, 0));
 
 	cdev_del(&esdm_cdev);
-	/* cdev_put(&esdm_cdev); */
 
 	unregister_chrdev_region(MKDEV(esdm_major, 0), ESDM_MAX_MINORS);
 	class_destroy(esdm_es_class);
@@ -185,29 +210,99 @@ static void __exit esdm_es_mgr_dev_fini(void)
 	pr_info("ESDM user space interface unavailable\n");
 }
 
+static int esdm_es_mgr_reboot_notifier(struct notifier_block *nb,
+				       unsigned long action, void *data)
+{
+	pr_warn("Reset ESDM ES' because of reboot/halt\n");
+
+	esdm_es_mgr_irq_reset();
+	esdm_es_mgr_sched_reset();
+
+	return NOTIFY_OK;
+}
+
+static struct notifier_block esdm_es_mgr_notifier_reboot = {
+	.notifier_call = esdm_es_mgr_reboot_notifier,
+	.priority = 0,
+};
+
+#ifdef CONFIG_VMGENID
+static int esdm_es_mgr_vmgenid_notifier(struct notifier_block *nb,
+					unsigned long action, void *data)
+{
+	pr_warn("Reset ESDM ES' because of vmgenid change!\n");
+
+	esdm_es_mgr_irq_reset();
+	esdm_es_mgr_sched_reset();
+
+	return NOTIFY_OK;
+}
+
+static struct notifier_block esdm_es_mgr_notifier_vmgenid = {
+	.notifier_call = esdm_es_mgr_vmgenid_notifier,
+	.priority = 0,
+};
+#endif
+
+static int esdm_es_mgr_power_notifier(struct notifier_block *nb,
+				      unsigned long action, void *data)
+{
+	pr_warn("Reset ESDM ES' because of power management change!\n");
+
+	esdm_es_mgr_irq_reset();
+	esdm_es_mgr_sched_reset();
+
+	return NOTIFY_OK;
+}
+
+static struct notifier_block esdm_es_mgr_notifier_power = {
+	.notifier_call = esdm_es_mgr_power_notifier,
+	.priority = 0,
+};
+
 static int __init esdm_es_mgr_init(void)
 {
 	int ret = esdm_init_time_source();
-
-	if (ret)
+	if (ret) {
+		pr_warn("esdm_init_time_source() failed\n");
 		goto out;
+	}
+
+	ret = esdm_drbg_selftest();
+	if (ret) {
+		pr_warn("esdm_drbg_selftest() failed\n");
+		goto out;
+	}
 
 	ret = esdm_es_mgr_irq_init();
-	if (ret)
+	if (ret) {
+		pr_warn("esdm_es_mgr_irq_init() failed\n");
 		goto out;
+	}
 
 	ret = esdm_es_mgr_sched_init();
-	if (ret)
+	if (ret) {
+		pr_warn("esdm_es_mgr_sched_init() failed\n");
 		goto out;
+	}
 
 	ret = esdm_test_init();
-	if (ret)
+	if (ret) {
+		pr_warn("esdm_test_init() failed\n");
 		goto out;
+	}
 
 	ret = esdm_es_mgr_dev_init();
-	if (ret)
+	if (ret) {
+		pr_warn("esdm_es_mgr_dev_init() failed\n");
 		goto out;
+	}
 
+	register_reboot_notifier(&esdm_es_mgr_notifier_reboot);
+#ifdef CONFIG_VMGENID
+	register_random_vmfork_notifier(&esdm_es_mgr_notifier_vmgenid);
+#endif
+	register_pm_notifier(&esdm_es_mgr_notifier_power);
 	return 0;
 
 out:
@@ -218,6 +313,12 @@ out:
 
 static void __exit esdm_es_mgr_exit(void)
 {
+	unregister_reboot_notifier(&esdm_es_mgr_notifier_reboot);
+#ifdef CONFIG_VMGENID
+	unregister_random_vmfork_notifier(&esdm_es_mgr_notifier_vmgenid);
+#endif
+	unregister_pm_notifier(&esdm_es_mgr_notifier_power);
+
 	esdm_es_mgr_dev_fini();
 	esdm_test_exit();
 	esdm_es_mgr_sched_exit();
