@@ -2,7 +2,7 @@
 /*
  * ESDM entropy source manager and external interface
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  */
 
 #define pr_fmt(fmt) KBUILD_MODNAME ": " fmt
@@ -17,7 +17,7 @@
 
 static u32 esdm_requested_sched_bits = ESDM_DRNG_INIT_SEED_SIZE_BITS;
 
-/* Available entropy in the entire ESDM considering all entropy sources */
+/* Available entropy in the scheduler ES considering all per-CPU pools */
 static u32 esdm_avail_entropy_sched(u32 requested_bits)
 {
 	return esdm_es_sched.curr_entropy(requested_bits);
@@ -68,6 +68,8 @@ int esdm_es_mgr_sched_ioctl(unsigned int cmd, unsigned long arg)
 		data &= ESDM_ES_MGR_REQ_BITS_MASK;
 		if (data > 0) {
 			if (data != ESDM_DRNG_INIT_SEED_SIZE_BITS &&
+			    data != (ESDM_DRNG_SECURITY_STRENGTH_BITS +
+				     ESDM_OVERSAMPLE_ES_BITS) &&
 			    data != ESDM_DRNG_SECURITY_STRENGTH_BITS)
 				return -EINVAL;
 			esdm_requested_sched_bits = data;
