@@ -2,7 +2,7 @@
 /*
  * ESDM Slow Entropy Source: Interrupt data collection
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  */
 
 #define pr_fmt(fmt) KBUILD_MODNAME ": " fmt
@@ -120,8 +120,12 @@ bool esdm_highres_timer(void)
 
 int __init esdm_init_time_source(void)
 {
-	if ((random_get_entropy() & ESDM_DATA_SLOTSIZE_MASK) ||
-	    (random_get_entropy() & ESDM_DATA_SLOTSIZE_MASK)) {
+	static const u32 ESDM_DATA_WORD_MASK = 0xFFFFFFFF;
+
+	/* only 32 bit of the timestamp are stored later,
+	 * check if they sufficiently change */
+	if ((random_get_entropy() & ESDM_DATA_WORD_MASK) ||
+	    (random_get_entropy() & ESDM_DATA_WORD_MASK)) {
 		/*
 		 * As the highres timer is identified here, previous interrupts
 		 * obtained during boot time are treated like a lowres-timer
@@ -133,8 +137,12 @@ int __init esdm_init_time_source(void)
 		esdm_highres_timer_val = false;
 	}
 
+#ifdef ESDM_ES_IRQ
 	esdm_irq_es_init(esdm_highres_timer_val);
+#endif /* ESDM_ES_IRQ */
+#ifdef ESDM_ES_SCHED
 	esdm_sched_es_init(esdm_highres_timer_val);
+#endif /* ESDM_ES_SCHED */
 
 	/* Ensure that changes to global variables are visible */
 	mb();
