@@ -2,7 +2,7 @@
 /*
  * Entropy Source and DRNG Manager (ESDM) Health Testing
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  */
 
 #define pr_fmt(fmt) KBUILD_MODNAME ": " fmt
@@ -10,20 +10,24 @@
 #include <linux/fips.h>
 #include <linux/module.h>
 
+#include "esdm_es_irq.h"
 #include "esdm_es_mgr.h"
 #include "esdm_es_mgr_cb.h"
+#include "esdm_es_sched.h"
 #include "esdm_health.h"
 
 /* Stuck Test */
 struct esdm_stuck_test {
-	u32 last_time; /* Stuck test: time of previous IRQ */
-	u32 last_delta; /* Stuck test: delta of previous IRQ */
-	u32 last_delta2; /* Stuck test: 2. time derivation of prev IRQ */
+	u32 last_time; /* Stuck test: time of previous event */
+	u32 last_delta; /* Stuck test: delta of previous event */
+	u32 last_delta2; /* Stuck test: 2. time derivation of previous event */
 };
 
 /* Repetition Count Test */
 struct esdm_rct {
 	atomic_t rct_count; /* Number of stuck values */
+	u16 cutoff;
+	u16 cutoff_permanent;
 };
 
 /* Adaptive Proportion Test */
@@ -37,6 +41,8 @@ struct esdm_apt {
 	atomic_t apt_base; /* APT base reference */
 
 	atomic_t apt_trigger;
+	u16 cutoff;
+	u16 cutoff_permanent;
 	bool apt_base_set; /* Is APT base set? */
 };
 
@@ -62,6 +68,12 @@ struct esdm_health_es_state {
 	x.sp80090b_startup_blocks = ATOMIC_INIT(ESDM_SP80090B_STARTUP_BLOCKS), \
 	x.sp80090b_startup_done = false,
 
+#define ESDM_HEALTH_OSR(curr) ((curr) / ESDM_DRNG_SECURITY_STRENGTH_BITS)
+#define ESDM_HEALTH_IRQ_RCT_CUTOFF(cutoff)                                     \
+	(ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) * cutoff)
+#define ESDM_HEALTH_SCHED_RCT_CUTOFF(cutoff)                                   \
+	(ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) * cutoff)
+
 /* The health test code must operate lock-less */
 struct esdm_health {
 	bool health_test_enabled;
@@ -73,9 +85,140 @@ static struct esdm_health esdm_health = {
 
 #ifdef ESDM_ES_IRQ
 	ESDM_HEALTH_ES_INIT(.es_state[esdm_int_es_irq])
+	.es_state[esdm_int_es_irq].rct.cutoff =
+		ESDM_HEALTH_IRQ_RCT_CUTOFF(CONFIG_ESDM_RCT_CUTOFF),
+	.es_state[esdm_int_es_irq].rct.cutoff_permanent =
+		ESDM_HEALTH_IRQ_RCT_CUTOFF(CONFIG_ESDM_RCT_CUTOFF_PERMANENT),
+#if (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 1)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_1,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_1,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 2)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_2,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_2,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 3)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_3,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_3,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 4)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_4,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_4,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 5)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_5,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_5,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 6)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_6,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_6,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 7)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_7,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_7,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 8)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_8,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_8,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 9)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_9,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_9,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 10)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_10,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_10,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 11)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_11,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_11,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 12)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_12,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_12,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 13)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_13,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_13,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_IRQ_ENTROPY_RATE) == 14)
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_14,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_14,
+#else
+	.es_state[esdm_int_es_irq].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_15,
+	.es_state[esdm_int_es_irq].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_15,
+#endif
+
 #endif
 #ifdef ESDM_ES_SCHED
-		ESDM_HEALTH_ES_INIT(.es_state[esdm_int_es_sched])
+	ESDM_HEALTH_ES_INIT(.es_state[esdm_int_es_sched])
+	.es_state[esdm_int_es_sched].rct.cutoff =
+		ESDM_HEALTH_SCHED_RCT_CUTOFF(CONFIG_ESDM_RCT_CUTOFF),
+	.es_state[esdm_int_es_sched].rct.cutoff_permanent =
+		ESDM_HEALTH_SCHED_RCT_CUTOFF(CONFIG_ESDM_RCT_CUTOFF_PERMANENT),
+#if (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 1)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_1,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_1,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 2)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_2,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_2,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 3)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_3,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_3,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 4)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_4,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_4,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 5)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_5,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_5,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 6)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_6,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_6,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 7)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_7,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_7,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 8)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_8,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_8,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 9)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_9,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_9,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 10)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_10,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_10,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 11)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_11,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_11,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 12)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_12,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_12,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 13)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_13,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_13,
+#elif (ESDM_HEALTH_OSR(CONFIG_ESDM_SCHED_ENTROPY_RATE) == 14)
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_14,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_14,
+#else
+	.es_state[esdm_int_es_sched].apt.cutoff = CONFIG_ESDM_APT_CUTOFF_15,
+	.es_state[esdm_int_es_sched].apt.cutoff_permanent =
+		CONFIG_ESDM_APT_CUTOFF_PERMANENT_15,
+#endif
 #endif
 };
 
@@ -98,16 +241,12 @@ static bool esdm_sp80090b_health_enabled(void)
 /***************************************************************************
  * SP800-90B Compliance
  *
- * If the Linux-RNG is booted into FIPS mode, the following interfaces
+ * If the esdm-es module is booted into FIPS mode, the following interfaces
  * provide an SP800-90B compliant noise source:
  *
- * * /dev/random
- * * getrandom(2)
- * * get_random_bytes_full
+ * - Scheduler ES
+ * - IRQ ES
  *
- * All other interfaces, including /dev/urandom or get_random_bytes without
- * the add_random_ready_callback cannot claim to use an SP800-90B compliant
- * noise source.
  ***************************************************************************/
 
 /*
@@ -214,11 +353,8 @@ bool esdm_sp80090b_startup_complete_es(enum esdm_internal_es es)
 
 bool esdm_sp80090b_compliant(enum esdm_internal_es es)
 {
-	struct esdm_health *health = &esdm_health;
-	struct esdm_health_es_state *es_state = &health->es_state[es];
-
 	return esdm_sp80090b_health_enabled() &&
-	       es_state->sp80090b_startup_done;
+	       esdm_sp80090b_startup_complete_es(es);
 }
 
 /***************************************************************************
@@ -276,9 +412,9 @@ static void esdm_apt_insert(struct esdm_health *health, unsigned int now_time,
 	if (now_time == (unsigned int)atomic_read(&apt->apt_base)) {
 		u32 apt_val = (u32)atomic_inc_return_relaxed(&apt->apt_count);
 
-		if (apt_val >= CONFIG_ESDM_APT_CUTOFF_PERMANENT)
+		if (apt_val >= apt->cutoff_permanent)
 			esdm_sp80090b_permanent_failure(health, es);
-		else if (apt_val >= CONFIG_ESDM_APT_CUTOFF)
+		else if (apt_val >= apt->cutoff)
 			esdm_sp80090b_failure(health, es);
 	}
 
@@ -341,9 +477,9 @@ static void esdm_rct(struct esdm_health *health, enum esdm_internal_es es,
 		 * Hence we need to subtract one from the cutoff value as
 		 * calculated following SP800-90B.
 		 */
-		if (rct_count >= CONFIG_ESDM_RCT_CUTOFF_PERMANENT)
+		if (rct_count > rct->cutoff_permanent)
 			esdm_sp80090b_permanent_failure(health, es);
-		else if (rct_count >= CONFIG_ESDM_RCT_CUTOFF)
+		else if (rct_count > rct->cutoff)
 			esdm_sp80090b_failure(health, es);
 	} else {
 		esdm_rct_reset(rct);
@@ -379,7 +515,7 @@ static u32 esdm_delta(u32 prev, u32 next)
  * @return: 0 event occurrence not stuck (good time stamp)
  *	    != 0 event occurrence stuck (reject time stamp)
  */
-static int esdm_irq_stuck(enum esdm_internal_es es, u32 now_time)
+static int esdm_timestamp_stuck(enum esdm_internal_es es, u32 now_time)
 {
 	struct esdm_stuck_test *stuck = this_cpu_ptr(esdm_stuck_test_array);
 	u32 delta = esdm_delta(stuck[es].last_time, now_time);
@@ -428,7 +564,7 @@ enum esdm_health_res esdm_health_test(u32 now_time, enum esdm_internal_es es)
 
 	esdm_apt_insert(health, now_time, es);
 
-	stuck = esdm_irq_stuck(es, now_time);
+	stuck = esdm_timestamp_stuck(es, now_time);
 	esdm_rct(health, es, stuck);
 	if (stuck) {
 		/* SP800-90B disallows using a failing health test time stamp */
