@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  */
 
 #ifndef _ESDM_TESTING_H
@@ -219,19 +219,7 @@ config ESDM_SCHED_PERF
 	  the first 1000 entropy events since boot can be sampled.
 */
 
-#undef CONFIG_ESDM_RAW_HIRES_ENTROPY
-#undef CONFIG_ESDM_RAW_JIFFIES_ENTROPY
-#undef CONFIG_ESDM_RAW_IRQ_ENTROPY
-#undef CONFIG_ESDM_RAW_RETIP_ENTROPY
-#undef CONFIG_ESDM_RAW_REGS_ENTROPY
-#undef CONFIG_ESDM_RAW_ARRAY
-#undef CONFIG_ESDM_IRQ_PERF
-
-#undef CONFIG_ESDM_RAW_SCHED_HIRES_ENTROPY
-#undef CONFIG_ESDM_RAW_SCHED_PID_ENTROPY
-#undef CONFIG_ESDM_RAW_SCHED_START_TIME_ENTROPY
-#undef CONFIG_ESDM_RAW_SCHED_NVCSW_ENTROPY
-#undef CONFIG_ESDM_SCHED_PERF
+/* definitions are located in Makefile now! */
 
 /******************************************************************************/
 
@@ -244,60 +232,6 @@ static inline bool esdm_raw_hires_entropy_store(u32 value)
 }
 #endif /* CONFIG_ESDM_RAW_HIRES_ENTROPY */
 
-#ifdef CONFIG_ESDM_RAW_JIFFIES_ENTROPY
-bool esdm_raw_jiffies_entropy_store(u32 value);
-#else /* CONFIG_ESDM_RAW_JIFFIES_ENTROPY */
-static inline bool esdm_raw_jiffies_entropy_store(u32 value)
-{
-	return false;
-}
-#endif /* CONFIG_ESDM_RAW_JIFFIES_ENTROPY */
-
-#ifdef CONFIG_ESDM_RAW_IRQ_ENTROPY
-bool esdm_raw_irq_entropy_store(u32 value);
-#else /* CONFIG_ESDM_RAW_IRQ_ENTROPY */
-static inline bool esdm_raw_irq_entropy_store(u32 value)
-{
-	return false;
-}
-#endif /* CONFIG_ESDM_RAW_IRQ_ENTROPY */
-
-#ifdef CONFIG_ESDM_RAW_RETIP_ENTROPY
-bool esdm_raw_retip_entropy_store(u32 value);
-#else /* CONFIG_ESDM_RAW_RETIP_ENTROPY */
-static inline bool esdm_raw_retip_entropy_store(u32 value)
-{
-	return false;
-}
-#endif /* CONFIG_ESDM_RAW_RETIP_ENTROPY */
-
-#ifdef CONFIG_ESDM_RAW_REGS_ENTROPY
-bool esdm_raw_regs_entropy_store(u32 value);
-#else /* CONFIG_ESDM_RAW_REGS_ENTROPY */
-static inline bool esdm_raw_regs_entropy_store(u32 value)
-{
-	return false;
-}
-#endif /* CONFIG_ESDM_RAW_REGS_ENTROPY */
-
-#ifdef CONFIG_ESDM_RAW_ARRAY
-bool esdm_raw_array_entropy_store(u32 value);
-#else /* CONFIG_ESDM_RAW_ARRAY */
-static inline bool esdm_raw_array_entropy_store(u32 value)
-{
-	return false;
-}
-#endif /* CONFIG_ESDM_RAW_ARRAY */
-
-#ifdef CONFIG_ESDM_IRQ_PERF
-bool esdm_perf_time(u32 start);
-#else /* CONFIG_ESDM_IRQ_PERF */
-static inline bool esdm_perf_time(u32 start)
-{
-	return false;
-}
-#endif /*CONFIG_ESDM_IRQ_PERF */
-
 #ifdef CONFIG_ESDM_RAW_SCHED_HIRES_ENTROPY
 bool esdm_raw_sched_hires_entropy_store(u32 value);
 #else /* CONFIG_ESDM_RAW_SCHED_HIRES_ENTROPY */
@@ -307,32 +241,14 @@ static inline bool esdm_raw_sched_hires_entropy_store(u32 value)
 }
 #endif /* CONFIG_ESDM_RAW_SCHED_HIRES_ENTROPY */
 
-#ifdef CONFIG_ESDM_RAW_SCHED_PID_ENTROPY
-bool esdm_raw_sched_pid_entropy_store(u32 value);
-#else /* CONFIG_ESDM_RAW_SCHED_PID_ENTROPY */
-static inline bool esdm_raw_sched_pid_entropy_store(u32 value)
+#ifdef CONFIG_ESDM_IRQ_PERF
+bool esdm_irq_perf_time(u32 start);
+#else /* CONFIG_ESDM_IRQ_PERF */
+static inline bool esdm_irq_perf_time(u32 start)
 {
 	return false;
 }
-#endif /* CONFIG_ESDM_RAW_SCHED_PID_ENTROPY */
-
-#ifdef CONFIG_ESDM_RAW_SCHED_START_TIME_ENTROPY
-bool esdm_raw_sched_starttime_entropy_store(u32 value);
-#else /* CONFIG_ESDM_RAW_SCHED_START_TIME_ENTROPY */
-static inline bool esdm_raw_sched_starttime_entropy_store(u32 value)
-{
-	return false;
-}
-#endif /* CONFIG_ESDM_RAW_SCHED_START_TIME_ENTROPY */
-
-#ifdef CONFIG_ESDM_RAW_SCHED_NVCSW_ENTROPY
-bool esdm_raw_sched_nvcsw_entropy_store(u32 value);
-#else /* CONFIG_ESDM_RAW_SCHED_NVCSW_ENTROPY */
-static inline bool esdm_raw_sched_nvcsw_entropy_store(u32 value)
-{
-	return false;
-}
-#endif /* CONFIG_ESDM_RAW_SCHED_NVCSW_ENTROPY */
+#endif /*CONFIG_ESDM_IRQ_PERF */
 
 #ifdef CONFIG_ESDM_SCHED_PERF
 bool esdm_sched_perf_time(u32 start);
