@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2018 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see COPYING file in root directory
  *
@@ -41,10 +41,7 @@ static inline void mb(void)
 	__sync_synchronize();
 }
 
-#define ATOMIC_INIT(i)                                                         \
-	{                                                                      \
-		(i)                                                            \
-	}
+#define ATOMIC_INIT(i) { (i) }
 
 /**
  * Read atomic variable
