@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -22,6 +22,8 @@
 
 #if defined(__s390__)
 
+#include <stdint.h>
+
 #include "bool.h"
 
 #define ESDM_CPU_ES_IMPLEMENTED
@@ -57,7 +59,7 @@ static inline void cpacf_trng(uint8_t *ucbuf, unsigned long ucbuf_len,
 
 static inline bool cpu_es_get(unsigned long *buf)
 {
-	cpacf_trng(NULL, 0, buf, sizeof(unsigned long));
+	cpacf_trng(NULL, 0, (uint8_t *)buf, sizeof(unsigned long));
 	return true;
 }
 
