@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -78,7 +78,7 @@ static inline int rdrand_available(void)
 
 	/* Only make call if the leaf is present */
 	if (eax >= 1) {
-		cpuid_eax(0x80000000 & 1, eax, ebx, ecx, edx);
+		cpuid_eax(1, eax, ebx, ecx, edx);
 		return !!(ecx & ECX_RDRAND);
 	}
 
