@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -80,7 +80,7 @@
 #define ESDM_INIT_ENTROPY_BITS 32
 
 /* AIS20/31: NTG.1.4 minimum entropy rate for one entropy source*/
-#define ESDM_AIS2031_NPTRNG_MIN_ENTROPY 220
+#define ESDM_AIS2031_NPTRNG_MIN_ENTROPY 240
 
 /*
  * Wakeup value
@@ -88,7 +88,7 @@
  * This value is allowed to be changed but must not be larger than the
  * digest size of the hash operation used update the aux_pool.
  */
-#define ESDM_WRITE_WAKEUP_ENTROPY SHA512_DIGEST_SIZE
+#define ESDM_WRITE_WAKEUP_ENTROPY (ESDM_NUM_AUX_POOLS * SHA512_DIGEST_SIZE)
 
 /*
  * Define the digest size for the conditioning components
