@@ -1,7 +1,7 @@
 /*
  * ESDM DRNG management
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -23,6 +23,7 @@
 #include <errno.h>
 #include <limits.h>
 #include <stdlib.h>
+#include <assert.h>
 
 #include "build_bug_on.h"
 #include "config.h"
@@ -151,10 +152,12 @@ void esdm_drng_reset(struct esdm_drng *drng)
 	/* Ensure reseed during next call */
 	atomic_set(&drng->requests, 1);
 	atomic_set(&drng->requests_since_fully_seeded, 0);
+	atomic_set(&drng->request_bits_since_fully_seeded, 0);
 	clock_gettime(CLOCK_MONOTONIC, &drng->last_seeded);
 	drng->fully_seeded = false;
 	/* Do not set force, as this flag is used for the emergency reseeding */
 	drng->force_reseed = false;
+	drng->initiated = false;
 	esdm_logger(LOGGER_DEBUG, LOGGER_C_DRNG, "reset DRNG\n");
 }
 
@@ -182,7 +185,7 @@ static void esdm_drng_dealloc_common(struct esdm_drng *drng)
 {
 	const struct esdm_drng_cb *drng_cb;
 
-	if (!drng)
+	if (!drng || !drng->drng)
 		return;
 
 	/* This only works with a robust mutex */
@@ -312,7 +315,7 @@ int esdm_ntg1_compliant(void)
 }
 
 DSO_PUBLIC
-int esdm_ntg1_2022_compliant(void)
+int esdm_ntg1_2024_compliant(void)
 {
 	return
 #ifdef ESDM_AIS2031_NTG1_SEEDING_STRATEGY
@@ -362,7 +365,8 @@ static time_t esdm_time_after_now(struct timespec *timeout)
 		return 0;
 
 	return esdm_time_after(&curr, timeout) ?
-			       (curr.tv_sec - timeout->tv_sec) : 0;
+		       (curr.tv_sec - timeout->tv_sec) :
+		       0;
 }
 
 /* Inject a data buffer into the DRNG - caller must hold its lock */
@@ -388,10 +392,15 @@ void esdm_drng_inject(struct esdm_drng *drng, const uint8_t *inbuf,
 			"%s DRNG stats since last seeding: %lu secs; generate calls: %d\n",
 			drng_type, esdm_time_after_now(&drng->last_seeded), gc);
 
-		/* Count the numbers of generate ops since last fully seeded */
-		if (fully_seeded)
+		/* Count the numbers of generate ops and output bits
+		 * since last fully seeded. The DRNG can be configured
+		 * to stop operation, if too many generate ops and/or output bits
+		 * were produced, without full reseeding again.
+		 */
+		if (fully_seeded) {
 			atomic_set(&drng->requests_since_fully_seeded, 0);
-		else
+			atomic_set(&drng->request_bits_since_fully_seeded, 0);
+		} else
 			atomic_add(&drng->requests_since_fully_seeded, gc);
 
 		clock_gettime(CLOCK_MONOTONIC, &drng->last_seeded);
@@ -400,10 +409,12 @@ void esdm_drng_inject(struct esdm_drng *drng, const uint8_t *inbuf,
 
 		if (!drng->fully_seeded) {
 			drng->fully_seeded = fully_seeded;
-			if (drng->fully_seeded)
+			if (drng->fully_seeded) {
+				drng->initiated = true;
 				esdm_logger(LOGGER_DEBUG, LOGGER_C_DRNG,
 					    "%s DRNG fully seeded\n",
 					    drng_type);
+			}
 		}
 	}
 }
@@ -422,6 +433,9 @@ static uint32_t esdm_drng_seed_es_nolock(struct esdm_drng *drng, bool init_ops,
 	uint32_t collected_entropy = 0;
 	unsigned int i, num_es_delivered = 0;
 	bool forced = drng->force_reseed;
+	unsigned int es_delivered_threshold = 1;
+	bool do_full_init = (drng == &esdm_drng_pr && !drng->initiated) ||
+			    (drng != &esdm_drng_pr && !drng->fully_seeded);
 
 	for_each_esdm_es (i)
 		collected_seedbuf.entropy_es[i].e_bits = 0;
@@ -432,6 +446,9 @@ static uint32_t esdm_drng_seed_es_nolock(struct esdm_drng *drng, bool init_ops,
 	 */
 	memset(&seedbuf, 0, sizeof(seedbuf));
 
+	if (esdm_ntg1_2024_compliant() && do_full_init)
+		es_delivered_threshold = 2;
+
 	do {
 		/* Count the number of ES which delivered entropy */
 		num_es_delivered = 0;
@@ -444,8 +461,8 @@ static uint32_t esdm_drng_seed_es_nolock(struct esdm_drng *drng, bool init_ops,
 		}
 
 		esdm_fill_seed_buffer(
-			&seedbuf, esdm_get_seed_entropy_osr(drng->fully_seeded),
-			forced && !drng->fully_seeded);
+			&seedbuf, esdm_get_seed_entropy_osr(do_full_init),
+			forced && do_full_init);
 
 		collected_entropy += esdm_entropy_rate_eb(&seedbuf);
 
@@ -458,7 +475,7 @@ static uint32_t esdm_drng_seed_es_nolock(struct esdm_drng *drng, bool init_ops,
 
 		/* Inject seed data into DRNG */
 		esdm_drng_inject(drng, (uint8_t *)&seedbuf, sizeof(seedbuf),
-				 esdm_fully_seeded(drng->fully_seeded,
+				 esdm_fully_seeded(do_full_init,
 						   collected_entropy,
 						   &collected_seedbuf),
 				 "regular");
@@ -472,7 +489,7 @@ static uint32_t esdm_drng_seed_es_nolock(struct esdm_drng *drng, bool init_ops,
 		if (init_ops)
 			esdm_init_ops(&collected_seedbuf);
 
-	/*
+		/*
 	 * Emergency reseeding: If we reached the min seed threshold now
 	 * multiple times but never reached fully seeded level and we collect
 	 * entropy, keep doing it until we reached fully seeded level for
@@ -486,18 +503,23 @@ static uint32_t esdm_drng_seed_es_nolock(struct esdm_drng *drng, bool init_ops,
 	 * producing data while this is ongoing.
 	 */
 	} while (forced && !drng->fully_seeded &&
-		 num_es_delivered >= (esdm_ntg1_2022_compliant() ? 2 : 1));
+		 num_es_delivered >= es_delivered_threshold);
 
 	memset_secure(&seedbuf, 0, sizeof(seedbuf));
 
 	return collected_entropy;
 }
 
-static void esdm_drng_seed_es(struct esdm_drng *drng)
+static void esdm_drng_seed_nolock(struct esdm_drng *drng)
 {
-	mutex_w_lock(&drng->lock);
+	BUILD_BUG_ON(ESDM_MIN_SEED_ENTROPY_BITS >
+		     ESDM_DRNG_SECURITY_STRENGTH_BITS);
+
+	/* (Re-)Seed DRNG */
 	esdm_drng_seed_es_nolock(drng, true, "regular");
-	mutex_w_unlock(&drng->lock);
+
+	/* (Re-)Seed atomic DRNG from regular DRNG */
+	esdm_drng_atomic_seed_drng(drng);
 }
 
 static void esdm_drng_seed(struct esdm_drng *drng)
@@ -506,7 +528,10 @@ static void esdm_drng_seed(struct esdm_drng *drng)
 		     ESDM_DRNG_SECURITY_STRENGTH_BITS);
 
 	/* (Re-)Seed DRNG */
-	esdm_drng_seed_es(drng);
+	mutex_w_lock(&drng->lock);
+	esdm_drng_seed_nolock(drng);
+	mutex_w_unlock(&drng->lock);
+
 	/* (Re-)Seed atomic DRNG from regular DRNG */
 	esdm_drng_atomic_seed_drng(drng);
 }
@@ -603,6 +628,28 @@ void esdm_drng_seed_work(void)
 	esdm_pool_unlock();
 }
 
+/**
+ * @brief Check if DRNG has to be temporarily disabled because of failed seedings
+ *
+ * @param [in] drng DRNG instance
+ *
+ * @return
+ * * true request or bit limit reached
+ * * false no disable condition triggered
+ */
+static bool esdm_drng_check_disable_threshold(struct esdm_drng *drng)
+{
+	bool request_limit_reached =
+		atomic_read_u32(&drng->requests_since_fully_seeded) >=
+		esdm_config_drng_max_wo_reseed();
+	bool bit_limit_reached =
+		(esdm_config_drng_max_wo_reseed_bits() != UINT32_MAX) &&
+		(atomic_read_u32(&drng->request_bits_since_fully_seeded) >=
+		 esdm_config_drng_max_wo_reseed_bits());
+
+	return request_limit_reached || bit_limit_reached;
+}
+
 /* Force all DRNGs to reseed before next generation */
 DSO_PUBLIC
 void esdm_drng_force_reseed(void)
@@ -615,9 +662,7 @@ void esdm_drng_force_reseed(void)
 	 * reseed only for the initial DRNG as this is the fallback for all. It
 	 * must be kept seeded before all others to keep the ESDM operational.
 	 */
-	if (!esdm_drng ||
-	    (atomic_read_u32(&esdm_drng_init.requests_since_fully_seeded) >
-	     ESDM_DRNG_RESEED_THRESH)) {
+	if (!esdm_drng || esdm_drng_check_disable_threshold(&esdm_drng_init)) {
 		esdm_drng_init.force_reseed = esdm_drng_init.fully_seeded;
 		esdm_logger(LOGGER_DEBUG, LOGGER_C_DRNG,
 			    "force reseed of initial DRNG\n");
@@ -641,12 +686,25 @@ out:
 	esdm_drng_put_instances();
 }
 
-static bool esdm_drng_must_reseed(struct esdm_drng *drng)
+static bool esdm_drng_must_reseed(struct esdm_drng *drng, bool dec_requests)
 {
 	struct timespec check_time = drng->last_seeded;
+	bool request_bits_since_fully_seeded_reached =
+		(ESDM_DRNG_RESEED_THRESH_BITS != UINT32_MAX) &&
+		(atomic_read_u32(&drng->request_bits_since_fully_seeded) >=
+		 ESDM_DRNG_RESEED_THRESH_BITS);
+	/*
+	 * atomic_dec_and_test will only trigger for zero, but we may also are
+	 * already negative
+	 */
+	bool requests_check = atomic_read(&drng->requests) <= 0;
 
+	if (dec_requests)
+		requests_check |= atomic_dec_and_test(&drng->requests);
 	check_time.tv_sec += esdm_drng_reseed_max_time;
-	return (atomic_dec_and_test(&drng->requests) || drng->force_reseed ||
+
+	return (requests_check || drng->force_reseed ||
+		request_bits_since_fully_seeded_reached ||
 		esdm_time_after_now(&check_time));
 }
 
@@ -664,7 +722,13 @@ static bool esdm_drng_must_reseed(struct esdm_drng *drng)
 static ssize_t esdm_drng_get(struct esdm_drng *drng, uint8_t *outbuf,
 			     size_t outbuflen)
 {
+	/* produce max. 128 Byte in pr mode without yielding */
+	const size_t pr_yield_iterations =
+		128 / (esdm_security_strength() >> 3);
+	/* should be greater than 1 for speedup */
+	assert(pr_yield_iterations > 1);
 	ssize_t processed = 0;
+	size_t iterations = 0;
 	bool pr = (drng == &esdm_drng_pr) ? true : false;
 
 	if (!outbuf || !outbuflen)
@@ -675,6 +739,29 @@ static ssize_t esdm_drng_get(struct esdm_drng *drng, uint8_t *outbuf,
 
 	outbuflen = min_size(outbuflen, SSIZE_MAX);
 
+	/*
+	 * try to reseed the last resort DRNG in a multi node setup
+	 * before unsetting the fully seeded state in the next check.
+	 *
+	 * This helps to get low but consistent performance when
+	 * operating in DRG.4 mode on request bursts
+	 * (e.g. in esdm-tool --benchmark).
+	 */
+	if (drng == esdm_drng_init_instance()) {
+		if (esdm_drng_must_reseed(drng, false)) {
+			esdm_pool_lock();
+
+			mutex_w_lock(&drng->lock);
+			/* double check, as we did not lock the DRNG in the first check*/
+			if (esdm_drng_must_reseed(drng, false)) {
+				esdm_drng_seed_nolock(drng);
+			}
+			mutex_w_unlock(&drng->lock);
+
+			esdm_pool_unlock();
+		}
+	}
+
 	/*
 	 * If the entire ESDM ran without full reseed for too long,
 	 * revert to the unseeded state.
@@ -683,8 +770,7 @@ static ssize_t esdm_drng_get(struct esdm_drng *drng, uint8_t *outbuf,
 	 * does not imply that sufficient entropy was received to fill the DRNG.
 	 * If this state persists, then the following check applies.
 	 */
-	if (atomic_read_u32(&drng->requests_since_fully_seeded) >
-	    esdm_config_drng_max_wo_reseed())
+	if (esdm_drng_check_disable_threshold(drng))
 		esdm_unset_fully_seeded(drng);
 
 	/* Loop to collect random bits for the caller. */
@@ -693,8 +779,10 @@ static ssize_t esdm_drng_get(struct esdm_drng *drng, uint8_t *outbuf,
 			min_uint32((uint32_t)outbuflen, ESDM_DRNG_MAX_REQSIZE);
 		ssize_t ret;
 
+		iterations++;
+
 		/* In normal operation, check whether to reseed */
-		if (!pr && esdm_drng_must_reseed(drng)) {
+		if (!pr && esdm_drng_must_reseed(drng, true)) {
 			if (!esdm_pool_trylock()) {
 				/*
 				 * Entropy pool cannot be locked, try to reseed
@@ -702,9 +790,13 @@ static ssize_t esdm_drng_get(struct esdm_drng *drng, uint8_t *outbuf,
 				 * bits.
 				 */
 				drng->force_reseed = true;
-			} else {
-				/* Perform synchronous reseed */
-				esdm_drng_seed(drng);
+			} else { /* Perform synchronous reseed */
+				mutex_w_lock(&drng->lock);
+				/* double check, as we did not lock the DRNG in the first check*/
+				if (esdm_drng_must_reseed(drng, false)) {
+					esdm_drng_seed_nolock(drng);
+				}
+				mutex_w_unlock(&drng->lock);
 				esdm_pool_unlock();
 			}
 		}
@@ -764,13 +856,15 @@ static ssize_t esdm_drng_get(struct esdm_drng *drng, uint8_t *outbuf,
 				ret);
 			return -EFAULT;
 		}
+		atomic_add(&drng->request_bits_since_fully_seeded,
+			   (int)ret << 3);
 		processed += ret;
 		outbuflen -= (size_t)ret;
 
 		if (pr) {
 			/* Force the async reseed for PR DRNG */
 			esdm_unset_fully_seeded(drng);
-			if (outbuflen)
+			if (outbuflen && iterations % pr_yield_iterations == 0)
 				sched_yield();
 		}
 	}
@@ -783,22 +877,56 @@ static ssize_t esdm_drng_get_sleep(uint8_t *outbuf, size_t outbuflen, bool pr)
 {
 	struct esdm_drng **esdm_drng = esdm_drng_get_instances();
 	struct esdm_drng *drng = &esdm_drng_init;
+	uint32_t num_nodes = esdm_config_online_nodes();
 	uint32_t node = esdm_config_curr_node();
+	bool found_drng = false;
 	ssize_t ret;
+	uint32_t j;
+	int32_t i;
 
 	if (pr) {
 		esdm_logger(
 			LOGGER_DEBUG, LOGGER_C_DRNG,
 			"Using prediction resistance DRNG instance to service generate request\n");
 		drng = &esdm_drng_pr;
-	} else if (esdm_drng && esdm_drng[node] &&
-		   esdm_drng[node]->fully_seeded) {
+		found_drng = true;
+	}
+
+	if (!found_drng && esdm_drng && esdm_drng[node] &&
+	    esdm_drng[node]->fully_seeded &&
+	    mutex_w_trylock(&esdm_drng[node]->lock)) {
+		mutex_w_unlock(&esdm_drng[node]->lock);
 		esdm_logger(
 			LOGGER_DEBUG, LOGGER_C_DRNG,
 			"Using DRNG instance on node %u to service generate request\n",
 			node);
 		drng = esdm_drng[node];
-	} else {
+		found_drng = true;
+	}
+
+	/*
+	 * try to find free drng heuristically first in order to better utilize
+	 * systems with RPC high load and a low total core count
+	 */
+	if (!found_drng && esdm_drng) {
+		for (i = (int32_t)num_nodes - 1; i >= 0; --i) {
+			/* every node starts probing at different offsets */
+			j = ((uint32_t)i + node) % num_nodes;
+			if (esdm_drng[j] && esdm_drng[j]->fully_seeded &&
+			    mutex_w_trylock(&drng->lock)) {
+				mutex_w_unlock(&drng->lock);
+				found_drng = true;
+				drng = esdm_drng[j];
+				esdm_logger(
+					LOGGER_DEBUG, LOGGER_C_DRNG,
+					"Using DRNG instance on node %u to service generate request\n",
+					j);
+				break;
+			}
+		}
+	}
+
+	if (!found_drng) {
 		esdm_logger(
 			LOGGER_DEBUG, LOGGER_C_DRNG,
 			"Using DRNG instance on node 0 to service generate request\n");
@@ -910,7 +1038,7 @@ static int esdm_drng_sleep_while_nonoperational_timeout(struct timespec *ts)
 		return 0;
 	thread_timedwait_event(&esdm_init_wait,
 			       esdm_state_operational() &&
-					!atomic_read(&esdm_drng_mgr_terminate),
+				       !atomic_read(&esdm_drng_mgr_terminate),
 			       ts);
 	return ret;
 }
@@ -976,8 +1104,8 @@ ssize_t esdm_get_seed(uint64_t *buf, size_t nbytes,
 	for (;;) {
 		esdm_fill_seed_buffer(
 			eb,
-			esdm_get_seed_entropy_osr(flags &
-						  ESDM_GET_SEED_FULLY_SEEDED),
+			esdm_get_seed_entropy_osr(!(flags &
+						  ESDM_GET_SEED_FULLY_SEEDED)),
 			false);
 		collected_bits = esdm_entropy_rate_eb(eb);
 
