@@ -1,7 +1,7 @@
 /*
  * ESDM Slow Entropy Source: Auxiliary entropy pool
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -19,12 +19,17 @@
  * DAMAGE.
  */
 
+#include <assert.h>
 #include <errno.h>
+#include <stdint.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 #include "config.h"
+#include "build_bug_on.h"
 #include "esdm.h"
 #include "esdm_crypto.h"
+#include "esdm_definitions.h"
 #include "esdm_es_aux.h"
 #include "esdm_es_mgr.h"
 #include "esdm_shm_status.h"
@@ -45,37 +50,57 @@
  * implementations.
  */
 struct esdm_pool {
-	void *aux_pool; /* Aux pool: digest state */
-	atomic_t aux_entropy_bits;
-	atomic_t digestsize; /* Digest size of used hash */
-	bool initialized; /* Aux pool initialized? */
+	/* Aux pool: digest state */
+	void *aux_pool;
 
 	/* Serialize read of entropy pool and update of aux pool */
 	mutex_w_t lock;
-};
 
-static struct esdm_pool esdm_pool __aligned(ESDM_KCAPI_ALIGN) = {
-	.aux_pool = NULL,
-	.aux_entropy_bits = ATOMIC_INIT(0),
-	.digestsize = ATOMIC_INIT(ESDM_MAX_DIGESTSIZE),
-	.initialized = false,
-	.lock = MUTEX_W_UNLOCKED,
-};
+	/* tracks entropy estimate in this pool */
+	atomic_t aux_entropy_bits;
+
+	/* Digest size of used hash */
+	atomic_t digestsize;
+
+	/* used for domain separation of inputs inserted into multiple pools */
+	uint16_t idx;
+
+	/* Aux pool initialized? */
+	bool initialized;
+} __aligned(ESDM_KCAPI_ALIGN);
+
+/*
+ * Global array of pools, use constant number of members
+ * such that compiler optimizations can take place
+ * e.g. eliminate loops, when only one pool is used.
+ */
+static struct esdm_pool esdm_pools[ESDM_NUM_AUX_POOLS] = { 0 };
 
 /********************************** Helper ***********************************/
 
 /* Entropy in bits present in aux pool */
-static uint32_t esdm_aux_avail_entropy(uint32_t __unused u)
+static uint32_t esdm_aux_avail_entropy_pool(struct esdm_pool *pool)
 {
 	/* Cap available entropy with max entropy */
 	uint32_t avail_bits =
 		min_uint32(esdm_get_digestsize(),
-			   atomic_read_u32(&esdm_pool.aux_entropy_bits));
+			   atomic_read_u32(&pool->aux_entropy_bits));
 
 	/* Consider oversampling rate due to aux pool conditioning */
 	return esdm_reduce_by_osr(avail_bits);
 }
 
+static uint32_t esdm_aux_avail_entropy(uint32_t __unused u)
+{
+	size_t i;
+	uint32_t avail_bits = 0;
+
+	for (i = 0; i < ESDM_NUM_AUX_POOLS; ++i)
+		avail_bits += esdm_aux_avail_entropy_pool(&esdm_pools[i]);
+
+	return avail_bits;
+}
+
 DSO_PUBLIC
 uint32_t esdm_get_aux_ent(void)
 {
@@ -83,20 +108,19 @@ uint32_t esdm_get_aux_ent(void)
 }
 
 /* Set the digest size of the used hash in bytes */
-static void esdm_set_digestsize(uint32_t digestsize)
+static void esdm_set_digestsize_pool(struct esdm_pool *pool,
+				     uint32_t digestsize)
 {
-	struct esdm_pool *pool = &esdm_pool;
 	uint32_t ent_bits = (uint32_t)atomic_xchg(&pool->aux_entropy_bits, 0),
 		 old_digestsize = esdm_get_digestsize();
 
-	atomic_set(&esdm_pool.digestsize, (int)digestsize);
+	atomic_set(&pool->digestsize, (int)digestsize);
 
 	/*
 	 * Update the write wakeup threshold which must not be larger
 	 * than the digest size of the current conditioning hash.
 	 */
 	digestsize = esdm_reduce_by_osr(digestsize << 3);
-	esdm_write_wakeup_bits = digestsize;
 
 	/*
 	 * In case the new digest is larger than the old one, cap the available
@@ -106,17 +130,26 @@ static void esdm_set_digestsize(uint32_t digestsize)
 	atomic_add(&pool->aux_entropy_bits, (int)ent_bits);
 }
 
-static void esdm_init_wakeup_bits(void)
+static void esdm_set_wakeup_bits(void)
 {
 	uint32_t digestsize = esdm_reduce_by_osr(esdm_get_digestsize());
 
-	esdm_write_wakeup_bits = digestsize;
+	esdm_write_wakeup_bits = ESDM_NUM_AUX_POOLS * digestsize;
 }
 
-static int esdm_aux_init(void)
+static void esdm_set_digestsize(uint32_t digestsize)
+{
+	size_t i;
+
+	for (i = 0; i < ESDM_NUM_AUX_POOLS; ++i)
+		esdm_set_digestsize_pool(&esdm_pools[i], digestsize);
+
+	esdm_set_wakeup_bits();
+}
+
+static int esdm_aux_init_pool(struct esdm_pool *pool)
 {
 	struct esdm_drng *drng = esdm_drng_init_instance();
-	struct esdm_pool *pool = &esdm_pool;
 	const struct esdm_hash_cb *hash_cb;
 	int ret = 0;
 
@@ -124,10 +157,40 @@ static int esdm_aux_init(void)
 	hash_cb = drng->hash_cb;
 	if (hash_cb->hash_alloc)
 		CKINT(hash_cb->hash_alloc(&pool->aux_pool));
-	esdm_logger(LOGGER_VERBOSE, LOGGER_C_ANY, "Aux ES hash allocated\n");
 	pool->initialized = false;
 
-	esdm_init_wakeup_bits();
+out:
+	mutex_unlock(&drng->hash_lock);
+	return ret;
+}
+
+static int esdm_aux_init(void)
+{
+	uint16_t i;
+	int ret = 0;
+
+	/* Sanity check for at least one pool */
+	BUILD_BUG_ON(ESDM_NUM_AUX_POOLS <= 0);
+
+	/* Sanity check to not overflow the idx variable */
+	BUILD_BUG_ON(ESDM_NUM_AUX_POOLS >
+		     (1 << (sizeof(esdm_pools[0].idx) << 3)) - 1);
+
+	/*
+	 * Initialize the member variables of the pool as we cannot do that
+	 * during compile time.
+	 */
+	for (i = 0; i < (uint16_t)ESDM_NUM_AUX_POOLS; ++i) {
+		esdm_pools[i].aux_pool = NULL;
+		atomic_set(&esdm_pools[i].aux_entropy_bits, 0);
+		atomic_set(&esdm_pools[i].digestsize, ESDM_MAX_DIGESTSIZE);
+		esdm_pools[i].initialized = false;
+		mutex_w_init(&esdm_pools[i].lock, 0, 0);
+		esdm_pools[i].idx = i;
+		esdm_aux_init_pool(&esdm_pools[i]);
+	}
+
+	esdm_set_wakeup_bits();
 
 	/*
 	 * Announce that the aux pool needs entropy - after startup, the aux
@@ -135,15 +198,14 @@ static int esdm_aux_init(void)
 	 */
 	esdm_shm_status_set_need_entropy();
 
-out:
-	mutex_unlock(&drng->hash_lock);
+	esdm_logger(LOGGER_VERBOSE, LOGGER_C_ANY, "Aux ES hash allocated\n");
+
 	return ret;
 }
 
-static void esdm_aux_fini(void)
+static void esdm_aux_fini_pool(struct esdm_pool *pool)
 {
 	struct esdm_drng *drng = esdm_drng_init_instance();
-	struct esdm_pool *pool = &esdm_pool;
 	const struct esdm_hash_cb *hash_cb;
 
 	mutex_lock(&drng->hash_lock);
@@ -151,28 +213,62 @@ static void esdm_aux_fini(void)
 	if (hash_cb->hash_dealloc)
 		hash_cb->hash_dealloc(pool->aux_pool);
 	pool->aux_pool = NULL;
-	esdm_logger(LOGGER_DEBUG, LOGGER_C_ANY, "Aux ES hash deallocated\n");
 	mutex_unlock(&drng->hash_lock);
 }
 
+static void esdm_aux_fini(void)
+{
+	size_t i;
+
+	for (i = 0; i < ESDM_NUM_AUX_POOLS; ++i) {
+		esdm_aux_fini_pool(&esdm_pools[i]);
+	}
+
+	esdm_logger(LOGGER_DEBUG, LOGGER_C_ANY, "Aux ES hash deallocated\n");
+}
+
 /* Obtain the digest size provided by the used hash in bits */
 DSO_PUBLIC
 uint32_t esdm_get_digestsize(void)
 {
-	return atomic_read_u32(&esdm_pool.digestsize) << 3;
+	if (esdm_pools[0].initialized)
+		return atomic_read_u32(&esdm_pools[0].digestsize) << 3;
+	else
+		return ESDM_MAX_DIGESTSIZE << 3;
+}
+
+static void esdm_pool_set_entropy_pool(struct esdm_pool *pool,
+				       uint32_t entropy_bits)
+{
+	atomic_set(&pool->aux_entropy_bits, (int)entropy_bits);
 }
 
 /* Set entropy content in user-space controllable aux pool */
 DSO_PUBLIC
 void esdm_pool_set_entropy(uint32_t entropy_bits)
 {
-	atomic_set(&esdm_pool.aux_entropy_bits, (int)entropy_bits);
+	size_t i;
+
+	/*
+	 * this interface can only influence the first pool
+	 * slot for security reasons, when entropy_bits > 0
+	 *
+	 * it can nevertheless reset all pools to zero
+	 */
+	for (i = 0; i < ESDM_NUM_AUX_POOLS; i++) {
+		esdm_pool_set_entropy_pool(&esdm_pools[i], entropy_bits);
+		if (entropy_bits != 0)
+			break;
+	}
 
 	/*
 	 * As the DRNG is newly seeded, maybe the need entropy flag can be
 	 * unset?
 	 */
 	esdm_shm_status_set_need_entropy();
+
+	/* notify others about newly added entropy */
+	esdm_es_add_entropy();
 }
 
 static void esdm_aux_reset(void)
@@ -186,16 +282,16 @@ static void esdm_aux_reset(void)
  * Assumption: the caller must guarantee that the new_cb is available during the
  * entire operation (e.g. it must hold the write lock against pointer updating).
  */
-static int esdm_aux_switch_hash(struct esdm_drng *drng, int __unused u,
-				const struct esdm_hash_cb *new_cb,
-				const struct esdm_hash_cb *old_cb)
+static int esdm_aux_switch_hash_pool(struct esdm_pool *pool,
+				     struct esdm_drng *drng,
+				     const struct esdm_hash_cb *new_cb,
+				     const struct esdm_hash_cb *old_cb)
 {
 #ifndef ESDM_CRYPTO_SWITCH
 	return -EOPNOTSUPP;
 #endif
 
 	struct esdm_drng *init_drng = esdm_drng_init_instance();
-	struct esdm_pool *pool = &esdm_pool;
 	void *shash = pool->aux_pool;
 	void *nhash = NULL;
 	uint8_t digest[ESDM_MAX_DIGESTSIZE];
@@ -215,11 +311,11 @@ static int esdm_aux_switch_hash(struct esdm_drng *drng, int __unused u,
 	/* ... re-initialize the hash with the new digest ... */
 	CKINT(new_cb->hash_init(nhash));
 	/*
-	       * ... feed the old hash into the new state. We may feed
-	       * uninitialized memory into the new state, but this is
-	       * considered no issue and even good as we have some more
-	       * uncertainty here.
-	       */
+	 * ... feed the old hash into the new state. We may feed
+	 * uninitialized memory into the new state, but this is
+	 * considered no issue and even good as we have some more
+	 * uncertainty here.
+	 */
 	CKINT(new_cb->hash_update(nhash, digest, sizeof(digest)));
 
 	/* Switch the hash state */
@@ -228,9 +324,6 @@ static int esdm_aux_switch_hash(struct esdm_drng *drng, int __unused u,
 	old_cb->hash_dealloc(shash);
 
 	esdm_set_digestsize(new_cb->hash_digestsize(pool->aux_pool));
-	esdm_logger(LOGGER_DEBUG, LOGGER_C_ES,
-		    "Re-initialize aux entropy pool with hash %s\n",
-		    new_cb->hash_name());
 
 out:
 	new_cb->hash_dealloc(nhash);
@@ -238,38 +331,71 @@ out:
 	return ret;
 }
 
+static int esdm_aux_switch_hash(struct esdm_drng *drng, int __unused u,
+				const struct esdm_hash_cb *new_cb,
+				const struct esdm_hash_cb *old_cb)
+{
+	size_t i;
+	int ret;
+
+	if (!esdm_pools[0].initialized)
+		return 0;
+
+	for (i = 0; i < ESDM_NUM_AUX_POOLS; ++i) {
+		CKINT(esdm_aux_switch_hash_pool(&esdm_pools[i], drng, new_cb,
+						old_cb));
+	}
+
+	esdm_logger(LOGGER_DEBUG, LOGGER_C_ES,
+		    "Re-initialize aux entropy pool with hash %s\n",
+		    new_cb->hash_name());
+
+out:
+	return ret;
+}
+
 /* Insert data into auxiliary pool by using the hash update function. */
-static int esdm_aux_pool_insert_locked(const uint8_t *inbuf, size_t inbuflen,
+static int esdm_aux_pool_insert_locked(struct esdm_pool *pool,
+				       const uint8_t *inbuf, size_t inbuflen,
 				       uint32_t entropy_bits)
 {
-	struct esdm_pool *pool = &esdm_pool;
 	struct hash_ctx *shash = (struct hash_ctx *)pool->aux_pool;
 	struct esdm_drng *drng = esdm_drng_init_instance();
 	const struct esdm_hash_cb *hash_cb;
 	int ret;
 
+	/* There can never be more entropy than the size of the input buffer. */
 	entropy_bits = min_uint32(entropy_bits, (uint32_t)(inbuflen << 3));
 
 	mutex_reader_lock(&drng->hash_lock);
 	hash_cb = drng->hash_cb;
 
 	if (!pool->initialized) {
-		ret = hash_cb->hash_init(shash);
-		if (ret)
-			goto out;
+		CKINT(hash_cb->hash_init(shash));
 		pool->initialized = true;
 	}
 
-	ret = hash_cb->hash_update(shash, inbuf, inbuflen);
-	if (ret)
-		goto out;
+	/*
+	 * Domain separation between pools: insert the index of the pool to
+	 * prevent any chance of identical states of the pool if the caller
+	 * might insert the same data into the pool.
+	 *
+	 * This implies that the actual data inserted into a pool is alway
+	 * idx || inbuf.
+	 */
+	CKINT(hash_cb->hash_update(shash, (const uint8_t *)&pool->idx,
+				   sizeof(pool->idx)));
+
+	/* Insert the actual data with or without entropy into the pool */
+	CKINT(hash_cb->hash_update(shash, inbuf, inbuflen));
 
 	/*
 	 * Cap the available entropy to the hash output size compliant to
 	 * SP800-90B section 3.1.5.1 table 1.
 	 */
 	entropy_bits += atomic_read_u32(&pool->aux_entropy_bits);
-	esdm_pool_set_entropy(
+	esdm_pool_set_entropy_pool(
+		pool,
 		min_uint32(entropy_bits, hash_cb->hash_digestsize(shash) << 3));
 
 out:
@@ -277,16 +403,94 @@ out:
 	return ret;
 }
 
+static int esdm_pool_insert_aux_unlocked(struct esdm_pool *pool,
+					 const uint8_t *inbuf, size_t inbuflen,
+					 uint32_t entropy_bits)
+{
+	int ret;
+
+	mutex_w_lock(&pool->lock);
+	ret = esdm_aux_pool_insert_locked(pool, inbuf, inbuflen, entropy_bits);
+	mutex_w_unlock(&pool->lock);
+
+	return ret;
+}
+
 DSO_PUBLIC
 int esdm_pool_insert_aux(const uint8_t *inbuf, size_t inbuflen,
 			 uint32_t entropy_bits)
 {
-	struct esdm_pool *pool = &esdm_pool;
+	size_t i, pool_with_max_entropy_capacity = 0;
+	uint32_t max_entropy_capacity = 0;
 	int ret;
 
-	mutex_w_lock(&pool->lock);
-	ret = esdm_aux_pool_insert_locked(inbuf, inbuflen, entropy_bits);
-	mutex_w_unlock(&pool->lock);
+	/*
+	 * Shortcut: if there is no entropy in the provided data, mix it into
+	 * all pools just to stir all of them.
+	 */
+	if (!entropy_bits) {
+		for (i = 0; i < ESDM_NUM_AUX_POOLS; ++i) {
+			CKINT(esdm_pool_insert_aux_unlocked(&esdm_pools[i],
+							    inbuf, inbuflen,
+							    entropy_bits));
+		}
+
+		/*
+		 * Return without any notifications as we did not add entropy or
+		 * change the entropy level.
+		 */
+		return 0;
+	}
+
+	/*
+	 * As of now, we don't do multi pool updates for security reasons, and
+	 * thus cap entropy to single pool capacity.
+	 */
+	if (entropy_bits > esdm_get_digestsize())
+		entropy_bits = esdm_get_digestsize();
+
+	/*
+	 * Now we want to find the pool to insert the entropy in. The applied
+	 * strategy is to find the pool that has the maximum amount of capacity
+	 * of yet unused entropy space or the pool that has sufficient capacity
+	 * to take our entropy.
+	 *
+	 * This approach shall ensure that we can store as much entropy of our
+	 * input block as possible.
+	 *
+	 * If all pools are full and do not have any capacity, the first pool
+	 * will receive the data.
+	 *
+	 * This strategy implies that we only insert the data into one pool
+	 * only. This may be considered inefficient (e.g. it may be nice to
+	 * spread the entropy over multiple pools if none of the existing pools
+	 * can take all entropy). But we deliberately do not apply this approach
+	 * as implementing it correctly without loosing entropy is not trivial.
+	 */
+	for (i = 0; i < ESDM_NUM_AUX_POOLS; ++i) {
+		uint32_t unused_ent =
+			esdm_get_digestsize() -
+			esdm_aux_avail_entropy_pool(&esdm_pools[i]);
+
+		/* Find the pool with the maximum capacity. */
+		if (max_entropy_capacity < unused_ent) {
+			max_entropy_capacity = unused_ent;
+			pool_with_max_entropy_capacity = i;
+		}
+
+		/* We found the pool that can already take all our entropy. */
+		if (entropy_bits < unused_ent)
+			break;
+	}
+
+	esdm_logger(LOGGER_DEBUG, LOGGER_C_ES,
+		    "Aux Pool %zu selected to insert data\n",
+		    pool_with_max_entropy_capacity);
+
+	/* Insert the buffer with the entropy into the selected entropy pool. */
+	CKINT(esdm_pool_insert_aux_unlocked(
+		&esdm_pools[pool_with_max_entropy_capacity], inbuf, inbuflen,
+		entropy_bits));
 
 	/*
 	 * As the DRNG is newly seeded, maybe the need entropy flag can be
@@ -294,6 +498,10 @@ int esdm_pool_insert_aux(const uint8_t *inbuf, size_t inbuflen,
 	 */
 	esdm_shm_status_set_need_entropy();
 
+	/* notify others about newly added entropy */
+	esdm_es_add_entropy();
+
+out:
 	return ret;
 }
 
@@ -306,9 +514,9 @@ int esdm_pool_insert_aux(const uint8_t *inbuf, size_t inbuflen,
  * @requested_bits: Requested amount of entropy
  * @return: amount of entropy in outbuf in bits.
  */
-static uint32_t esdm_aux_get_pool(uint8_t *outbuf, uint32_t requested_bits)
+static uint32_t esdm_aux_get_pool(struct esdm_pool *pool, uint8_t *outbuf,
+				  uint32_t requested_bits)
 {
-	struct esdm_pool *pool = &esdm_pool;
 	struct hash_ctx *shash = (struct hash_ctx *)pool->aux_pool;
 	struct esdm_drng *drng = esdm_drng_init_instance();
 	const struct esdm_hash_cb *hash_cb;
@@ -382,20 +590,73 @@ static uint32_t esdm_aux_get_pool(uint8_t *outbuf, uint32_t requested_bits)
 static void esdm_aux_get_backtrack(struct entropy_es *eb_es,
 				   uint32_t requested_bits, bool __unused u)
 {
-	struct esdm_pool *pool = &esdm_pool;
+	size_t i, pool_with_max_entropy = 0;
+	uint32_t max_entropy = 0;
+	bool locked_pool = false;
 
-	/* Ensure aux pool extraction and backtracking op are atomic */
-	mutex_w_lock(&pool->lock);
+	/*
+	 * Now we want to find the pool to extract the entropy from. The applied
+	 * strategy is to find the pool that has the maximum amount of entropy
+	 * or the pool that has sufficient entropy to satisfy the request.
+	 *
+	 * Yet, we are only pulling data from one entropy pool (and do not)
+	 * compress the data from multiple pools to satisfy the requested
+	 * entropy. This approach currently is taken for efficiency reasons, but
+	 * can easily be revised in the future.
+	 *
+	 * If all pools are empty and do not have any entropy, the first pool
+	 * will be used for the request and let the actual extraction function
+	 * handle the situation where there is no entropy.
+	 */
+	for (i = 0; i < ESDM_NUM_AUX_POOLS; ++i) {
+		/* Ensure aux pool level lookup, extraction and backtracking op are atomic */
+		mutex_w_lock(&esdm_pools[i].lock);
 
-	eb_es->e_bits = esdm_aux_get_pool(eb_es->e, requested_bits);
+		uint32_t avail_ent =
+			esdm_aux_avail_entropy_pool(&esdm_pools[i]);
+
+		/* Find the maximum entropy */
+		if (max_entropy < avail_ent) {
+			max_entropy = avail_ent;
+			pool_with_max_entropy = i;
+		}
+
+		/*
+		 * We found the pool that can already provide all our entropy
+		 * needs (including the discount for the OSR), take it.
+		 */
+		if (requested_bits + esdm_compress_osr() <= max_entropy) {
+			locked_pool = true;
+			break;
+		}
+
+		mutex_w_unlock(&esdm_pools[i].lock);
+	}
+
+	if (!locked_pool)
+		mutex_w_lock(&esdm_pools[pool_with_max_entropy].lock);
+
+	/* Now we have the pool that we want to extract data from. */
+	esdm_logger(LOGGER_DEBUG, LOGGER_C_ES,
+		    "Aux Pool %zu selected to obtain data\n",
+		    pool_with_max_entropy);
+
+	eb_es->e_bits = esdm_aux_get_pool(&esdm_pools[pool_with_max_entropy],
+					  eb_es->e, requested_bits);
 
 	/* Mix the extracted data back into pool for backtracking resistance */
-	if (esdm_aux_pool_insert_locked((uint8_t *)eb_es,
+	if (esdm_aux_pool_insert_locked(&esdm_pools[pool_with_max_entropy],
+					(uint8_t *)eb_es,
 					sizeof(struct entropy_es), 0))
 		esdm_logger(LOGGER_WARN, LOGGER_C_ES,
 			    "Backtracking resistance operation failed\n");
 
-	mutex_w_unlock(&pool->lock);
+	mutex_w_unlock(&esdm_pools[pool_with_max_entropy].lock);
+}
+
+static uint32_t esdm_aux_max_entropy(void)
+{
+	return esdm_get_digestsize() * ESDM_NUM_AUX_POOLS;
 }
 
 static void esdm_aux_es_state(char *buf, size_t buflen)
@@ -405,9 +666,15 @@ static void esdm_aux_es_state(char *buf, size_t buflen)
 	/* Assume the esdm_drng_init lock is taken by caller */
 	snprintf((char *)buf, buflen,
 		 " Hash for operating entropy pool: %s\n"
-		 " Available entropy: %u\n",
+		 " Available entropy: %u\n"
+		 " Maximum entropy: %u\n"
+		 " Pools: %u\n"
+		 " Write wakeup threshold: %u\n"
+		 " Digestsize: %u",
 		 esdm_drng_init->hash_cb->hash_name(),
-		 esdm_aux_avail_entropy(0));
+		 esdm_aux_avail_entropy(0), esdm_aux_max_entropy(),
+		 ESDM_NUM_AUX_POOLS, esdm_write_wakeup_bits,
+		 esdm_get_digestsize());
 }
 
 static bool esdm_aux_active(void)
@@ -422,7 +689,7 @@ struct esdm_es_cb esdm_es_aux = {
 	.monitor_es = NULL,
 	.get_ent = esdm_aux_get_backtrack,
 	.curr_entropy = esdm_aux_avail_entropy,
-	.max_entropy = esdm_get_digestsize,
+	.max_entropy = esdm_aux_max_entropy,
 	.state = esdm_aux_es_state,
 	.reset = esdm_aux_reset,
 	.active = esdm_aux_active,
