@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -42,12 +42,12 @@ static inline uint32_t esdm_security_strength(void)
 	return min_uint32(ESDM_FULL_SEED_ENTROPY_BITS, esdm_get_digestsize());
 }
 
-static inline uint32_t esdm_get_seed_entropy_osr(bool fully_seeded)
+static inline uint32_t esdm_get_seed_entropy_osr(bool do_full_init)
 {
 	uint32_t requested_bits = esdm_security_strength();
 
 	/* Apply oversampling during initialization according to SP800-90C */
-	if (esdm_sp80090c_compliant() && !fully_seeded)
+	if (esdm_sp80090c_compliant() && do_full_init)
 		requested_bits += ESDM_SEED_BUFFER_INIT_ADD_BITS;
 	return requested_bits;
 }
