@@ -1,7 +1,7 @@
 /*
  * ESDM Slow Entropy Source: Interrupt-based
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -27,9 +27,8 @@
 
 #include "esdm_config.h"
 #include "esdm_es_aux.h"
-#include "esdm_es_mgr.h"
 #include "esdm_es_irq.h"
-#include "esdm_es_sched.h"
+#include "esdm_es_mgr.h"
 #include "helper.h"
 #include "esdm_logger.h"
 #include "memset_secure.h"
@@ -139,10 +138,20 @@ static int esdm_irq_initialize(void)
 	}
 
 	ret = ioctl(fd, ESDM_IRQ_ENT_BUF_SIZE, &status);
+	if (ret < 0 && errno == ENOTTY) {
+		esdm_logger(
+			esdm_config_es_irq_retry() ? LOGGER_VERBOSE :
+						     LOGGER_WARN,
+			LOGGER_C_ES,
+			"Disabling interrupt-based entropy source which is not present in kernel\n");
+		esdm_irq_finalize();
+		return 0;
+	}
 	if (ret < 0) {
 		esdm_logger(
 			LOGGER_ERR, LOGGER_C_ES,
-			"Failure to obtain interrupt entropy source status from kernel\n");
+			"Failure to obtain interrupt entropy source status from kernel, errno: %i, error: %s\n",
+			errno, strerror(errno));
 		close(fd);
 		return -EAGAIN;
 	}
@@ -285,7 +294,10 @@ static void esdm_irq_es_state(char *buf, size_t buflen)
 			snprintf(buf, buflen,
 				 " failure in reading kernel status\n");
 		} else if (buflen < sizeof(status)) {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-truncation"
 			snprintf(buf, buflen, "%s", status_p);
+#pragma GCC diagnostic pop
 		}
 	} else {
 		snprintf(buf, buflen, " disabled - missing kernel support\n");
