@@ -1,7 +1,7 @@
 /*
  * ESDM Fast Entropy Source: Jitter RNG
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -127,7 +127,7 @@ err:
 
 static int esdm_jent_async_monitor(void)
 {
-	unsigned int i, requested_bits = esdm_get_seed_entropy_osr(true);
+	unsigned int i, requested_bits = esdm_get_seed_entropy_osr(false);
 
 	if (!esdm_config_es_jent_async_enabled())
 		return 0;
@@ -142,7 +142,7 @@ static int esdm_jent_async_monitor(void)
 			continue;
 
 		/*
-		 * Always gather entropy data including
+		 * Always gather entropy data excluding
 		 * potential oversampling factor.
 		 */
 		esdm_jent_get(esdm_jent_state_thread, &esdm_jent_async[i],
@@ -217,8 +217,9 @@ static void esdm_jent_async_get(struct entropy_es *eb_es,
 static void esdm_jent_get_check(struct entropy_es *eb_es,
 				uint32_t requested_bits, bool __unused unused)
 {
+	/* serve small requests without initial oversampling from async cache */
 	if (esdm_config_es_jent_async_enabled() &&
-	    (requested_bits == esdm_get_seed_entropy_osr(true))) {
+	    (requested_bits == esdm_get_seed_entropy_osr(false))) {
 		esdm_jent_async_get(eb_es, requested_bits, unused);
 	} else {
 		mutex_w_lock(&esdm_jent_lock);
@@ -256,7 +257,9 @@ static void esdm_jent_async_fini(void)
 static void esdm_jent_get_check(struct entropy_es *eb_es,
 				uint32_t requested_bits, bool __unused unused)
 {
-	esdm_jent_get(eb_es, requested_bits, unused);
+	mutex_w_lock(&esdm_jent_lock);
+	esdm_jent_get(esdm_jent_state, eb_es, requested_bits, unused);
+	mutex_w_unlock(&esdm_jent_lock);
 }
 
 static inline int esdm_jent_async_init(void)
@@ -289,6 +292,7 @@ static void esdm_jent_finalize(void)
 
 static int esdm_jent_initialize(void)
 {
+	unsigned int flags = 0;
 	int ret;
 
 	/* Allow the init function to be called multiple times */
@@ -301,7 +305,20 @@ static int esdm_jent_initialize(void)
 	/* Initialize the Jitter RNG after the clocksources are initialized. */
 	CKINT(esdm_jent_async_init());
 
-	esdm_jent_state = jent_entropy_collector_alloc(0, 0);
+	if (esdm_config_sp80090c_compliant() || esdm_config_fips_enabled()) {
+		flags |= JENT_FORCE_FIPS;
+	}
+
+	if (esdm_ntg1_2024_compliant()) {
+#if JENT_VERSION >= 3070000
+		flags |= JENT_NTG1;
+#endif
+		/* should also be set by JENT_NTG1, just set as
+		 * part of defensive programming */
+		flags |= JENT_FORCE_FIPS;
+	}
+
+	esdm_jent_state = jent_entropy_collector_alloc(0, flags);
 	CKNULL(esdm_jent_state, -EFAULT);
 
 	atomic_set(&esdm_jent_initialized, 1);
@@ -322,11 +339,22 @@ out:
 
 static void esdm_jent_es_state(char *buf, size_t buflen)
 {
+#if JENT_VERSION >= 3070000
+	const bool jent_secure_memory = jent_secure_memory_supported();
+#else
+	const bool jent_secure_memory = false;
+#endif
+
 	snprintf(buf, buflen,
 		 " Available entropy: %u\n"
 		 " Library version: %u\n"
+		 " Standards compliance: %s%s\n"
 		 " Entropy Rate per 256 data bits: %u\n",
 		 esdm_jent_poolsize(), jent_version(),
+		 (esdm_sp80090c_compliant() ||
+		  esdm_config_fips_enabled() ||
+		  esdm_ntg1_2024_compliant()) ? "SP800-90B " : "",
+		 (esdm_ntg1_2024_compliant() && jent_version() >= 3070000 && jent_secure_memory) ? "NTG.1(2024)" : "",
 		 esdm_jent_entropylevel(256));
 }
 
