@@ -1,7 +1,7 @@
 /*
  * ESDM Entropy sources management
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -20,6 +20,7 @@
  */
 
 #define _POSIX_C_SOURCE 200112L
+#include <assert.h>
 #include <errno.h>
 #include <stdint.h>
 #include <sys/ioctl.h>
@@ -432,11 +433,15 @@ static uint32_t esdm_avail_entropy_thresh(void)
 	return ent_thresh;
 }
 
-bool esdm_fully_seeded(bool fully_seeded, uint32_t collected_entropy,
+bool esdm_fully_seeded(bool do_full_init, uint32_t collected_entropy,
 		       struct entropy_buf *eb)
 {
-	/* AIS20/31 NTG.1: two entropy sources with each delivering 220 bits */
-	if (esdm_ntg1_2022_compliant()) {
+	/*
+	 * AIS20/31 NTG.1:
+	 * - Two entropy sources with each delivering 240 bits initially
+	 * - After the initial seeding step one entropy source is sufficient
+	 */
+	if (do_full_init && esdm_ntg1_2024_compliant()) {
 		uint32_t i, result = 0,
 			    ent_thresh = esdm_avail_entropy_thresh();
 
@@ -449,7 +454,7 @@ bool esdm_fully_seeded(bool fully_seeded, uint32_t collected_entropy,
 		return (result >= 2);
 	}
 
-	return (collected_entropy >= esdm_get_seed_entropy_osr(fully_seeded));
+	return (collected_entropy >= esdm_get_seed_entropy_osr(do_full_init));
 }
 
 uint32_t esdm_entropy_rate_eb(struct entropy_buf *eb)
@@ -552,11 +557,11 @@ void esdm_set_write_wakeup_bits(uint32_t val)
 
 static uint32_t esdm_init_entropy_level(bool fully_seeded)
 {
-	return esdm_ntg1_2022_compliant() ?
-		       /* Approximation so that two ES should deliver 220 bits each */
+	return esdm_ntg1_2024_compliant() ?
+		       /* Approximation so that two ES should deliver 240 bits each */
 		       (2 * ESDM_AIS2031_NPTRNG_MIN_ENTROPY) :
 		       /* Apply SP800-90C oversampling if applicable */
-		       esdm_get_seed_entropy_osr(fully_seeded);
+		       esdm_get_seed_entropy_osr(!fully_seeded);
 }
 
 /**
@@ -595,7 +600,7 @@ void esdm_init_ops(struct entropy_buf *eb)
 	if (state->esdm_fully_seeded) {
 		esdm_set_operational();
 		esdm_set_entropy_thresh(requested_bits);
-	} else if (esdm_fully_seeded(state->all_online_nodes_seeded, seed_bits,
+	} else if (esdm_fully_seeded(!state->all_online_nodes_seeded, seed_bits,
 				     eb)) {
 		state->esdm_fully_seeded = true;
 		esdm_set_operational();
@@ -771,12 +776,14 @@ void esdm_fill_seed_buffer(struct entropy_buf *eb, uint32_t requested_bits,
 	uint32_t i, req_ent = esdm_sp80090c_compliant() ?
 				      esdm_security_strength() :
 				      ESDM_MIN_SEED_ENTROPY_BITS;
+	int ret;
 
 	/* Guarantee that requested bits is a multiple of bytes */
 	BUILD_BUG_ON(ESDM_DRNG_SECURITY_STRENGTH_BITS % 8);
 
 	/* always reseed the DRNG with the current time stamp */
-	eb->now = time(NULL);
+	ret = clock_gettime(CLOCK_MONOTONIC, &eb->now);
+	assert(ret == 0);
 
 	/*
 	 * Require at least 128 bits of entropy for any reseed. If the ESDM is
