@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  */
 
 #ifndef _ESDM_ES_MGR_H
@@ -36,7 +36,7 @@ void esdm_pool_unlock(void);
 void esdm_pool_all_nodes_seeded(bool set);
 bool esdm_pool_all_nodes_seeded_get(void);
 
-bool esdm_fully_seeded(bool fully_seeded, uint32_t collected_entropy,
+bool esdm_fully_seeded(bool do_full_init, uint32_t collected_entropy,
 		       struct entropy_buf *eb);
 uint32_t esdm_entropy_rate_eb(struct entropy_buf *eb);
 void esdm_unset_fully_seeded(struct esdm_drng *drng);
