@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * Definition of an entropy source.
  *
@@ -90,7 +90,7 @@ struct entropy_es {
 
 struct entropy_buf {
 	struct entropy_es entropy_es[esdm_ext_es_last];
-	time_t now;
+	struct timespec now;
 };
 
 /*
