@@ -1,7 +1,7 @@
 /*
  * ESDM Slow Entropy Source: Scheduler-based
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -27,8 +27,8 @@
 
 #include "esdm_config.h"
 #include "esdm_es_aux.h"
-#include "esdm_es_mgr.h"
 #include "esdm_es_sched.h"
+#include "esdm_es_mgr.h"
 #include "helper.h"
 #include "esdm_logger.h"
 #include "memset_secure.h"
@@ -135,14 +135,25 @@ static int esdm_sched_initialize(void)
 			esdm_config_es_sched_retry() ? LOGGER_VERBOSE :
 						       LOGGER_WARN,
 			LOGGER_C_ES,
-			"Disabling scheduler-based entropy source which is not present in kernel\n") return 0;
+			"Disabling scheduler-based entropy source which is not present in kernel\n");
+		return 0;
 	}
 
 	ret = ioctl(fd, ESDM_SCHED_ENT_BUF_SIZE, &status);
+	if (ret < 0 && errno == ENOTTY) {
+		esdm_logger(
+			esdm_config_es_sched_retry() ? LOGGER_VERBOSE :
+						       LOGGER_WARN,
+			LOGGER_C_ES,
+			"Disabling scheduler-based entropy source which is not present in kernel\n");
+		esdm_sched_finalize();
+		return 0;
+	}
 	if (ret < 0) {
 		esdm_logger(
 			LOGGER_ERR, LOGGER_C_ES,
-			"Failure to obtain scheduler entropy source status from kernel\n");
+			"Failure to obtain scheduler entropy source status from kernel, errno: %i, error: %s\n",
+			errno, strerror(errno));
 		close(fd);
 		return -EAGAIN;
 	}
@@ -276,7 +287,10 @@ static void esdm_sched_es_state(char *buf, size_t buflen)
 			snprintf(buf, buflen,
 				 " failure in reading kernel status\n");
 		} else if (buflen < sizeof(status)) {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-truncation"
 			snprintf(buf, buflen, "%s", status_p);
+#pragma GCC diagnostic pop
 		}
 	} else {
 		snprintf(buf, buflen, " disabled - missing kernel support\n");
