@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -460,7 +460,7 @@ void esdm_cuse_read_internal(fuse_req_t req, size_t size, off_t off,
 			     struct fuse_file_info *fi, get_func_t get,
 			     int fallback_fd)
 {
-	uint8_t tmpbuf_s[64], *tmpbuf_p = tmpbuf_s, *tmpbuf = NULL;
+	uint8_t tmpbuf_s[64] = { 0 }, *tmpbuf_p = tmpbuf_s, *tmpbuf = NULL;
 	size_t read_bytes = 0;
 	ssize_t ret = 0;
 
@@ -796,6 +796,29 @@ void esdm_cuse_ioctl(int backend_fd, fuse_req_t req, unsigned long cmd,
 		}
 		break;
 
+	/* ESDM-specific IOCTL: force reseed of kernel CRNG directly */
+	case 44:
+		/*
+		* This operation requires privileges. Thus, raise the
+		* privilege level to the same level as the caller has.
+		*/
+		if (!esdm_cuse_client_privileged(req)) {
+			fuse_reply_err(req, EPERM);
+			return;
+		}
+		esdm_cuse_raise_privilege_transient(req);
+		if (backend_fd >= 0 &&
+			ioctl(backend_fd, RNDRESEEDCRNG) == -1)
+			ret = -errno;
+		else
+			ret = 0;
+		esdm_cuse_drop_privilege_transient();
+		if (ret)
+			fuse_reply_err(req, -ret);
+		else
+			fuse_reply_ioctl(req, 0, NULL, 0);
+		break;
+
 	default:
 		fuse_reply_err(req, EINVAL);
 	}
@@ -1026,12 +1049,10 @@ struct esdm_cuse_param {
 	unsigned int verbosity;
 	int is_help;
 	int disable_fallback;
+	int syslog;
 };
 
-#define ESDM_CUSE_OPT(t, p)                                                    \
-	{                                                                      \
-		t, offsetof(struct esdm_cuse_param, p), 1                      \
-	}
+#define ESDM_CUSE_OPT(t, p) { t, offsetof(struct esdm_cuse_param, p), 1 }
 
 static const char *usage =
 	"usage: esdm_cuse [options]\n"
@@ -1046,6 +1067,7 @@ static const char *usage =
 	"    -d   -o debug           enable debug output (implies -f)\n"
 	"    -f                      foreground operation\n"
 	"    -s                      disable multi-threaded operation\n"
+	"    -S                      log via syslog\n"
 	"\n";
 
 /* The CUSE code seems to have a conversion issue with FUSE_OPT_KEY */
@@ -1069,6 +1091,8 @@ static const struct fuse_opt esdm_cuse_opts[] = {
 #endif
 	FUSE_OPT_KEY("-h", 0),
 	FUSE_OPT_KEY("--help", 0),
+	FUSE_OPT_KEY("-S", 1),
+	FUSE_OPT_KEY("--syslog", 1),
 	FUSE_OPT_END
 };
 #pragma GCC diagnostic pop
@@ -1086,6 +1110,9 @@ static int esdm_cuse_process_arg(void *data, const char *arg, int key,
 		param->is_help = 1;
 		fprintf(stderr, "%s", usage);
 		return fuse_opt_add_arg(outargs, "-ho");
+	case 1:
+		param->syslog = 1;
+		return 0;
 	default:
 		return 1;
 	}
@@ -1095,7 +1122,7 @@ int main_common(const char *_devname, const char *target, const char *semname,
 		const struct cuse_lowlevel_ops *clop, int argc, char **argv)
 {
 	struct fuse_args args = FUSE_ARGS_INIT(argc, argv);
-	struct esdm_cuse_param param = { 0, 0, NULL, NULL, 1, 0, 0 };
+	struct esdm_cuse_param param = { 0, 0, NULL, NULL, 1, 0, 0, 0 };
 	char dev_name[128] = "DEVNAME=";
 	char devname[20];
 	const char *dev_info_argv[] = { dev_name };
@@ -1112,6 +1139,10 @@ int main_common(const char *_devname, const char *target, const char *semname,
 
 	esdm_logger_set_verbosity(param.verbosity);
 
+	if (param.syslog) {
+		esdm_logger_enable_syslog("esdm-cuse");
+	}
+
 	esdm_test_disable_fallback(param.disable_fallback);
 
 	CKINT(esdm_cuse_file_name(devname, sizeof(devname), _devname));
