@@ -1,6 +1,6 @@
 /* ESDM /proc/sys/kernel/random interface
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -570,16 +570,18 @@ static struct esdm_proc_options {
 	int show_help;
 	int relabel;
 	unsigned int verbosity;
+	int syslog;
 } esdm_proc_options;
 
-#define OPTION(t, p)                                                           \
-	{                                                                      \
-		t, offsetof(struct esdm_proc_options, p), 1                    \
-	}
+#define OPTION(t, p) { t, offsetof(struct esdm_proc_options, p), 1 }
 static const struct fuse_opt esdm_proc_options_spec[] = {
-	OPTION("-v %u", verbosity),   OPTION("--verbosity=%u", verbosity),
-	OPTION("--relabel", relabel), OPTION("-h", show_help),
-	OPTION("--help", show_help),  FUSE_OPT_END
+	OPTION("-v %u", verbosity),
+	OPTION("--verbosity=%u", verbosity),
+	OPTION("--relabel", relabel),
+	OPTION("-h", show_help),
+	OPTION("--syslog", syslog),
+	OPTION("--help", show_help),
+	FUSE_OPT_END
 };
 
 static void show_help(const char *progname)
@@ -588,6 +590,7 @@ static void show_help(const char *progname)
 	printf("File-system specific options:\n"
 	       "    --verbosity=<u>     Verbosity level\n"
 	       "    --relabel           Perform automatic SELinux relabeling"
+	       "    --syslog            Log via syslog"
 	       "\n");
 }
 
@@ -605,6 +608,10 @@ int main(int argc, char *argv[])
 
 	esdm_logger_set_verbosity(esdm_proc_options.verbosity);
 
+	if (esdm_proc_options.syslog) {
+		esdm_logger_enable_syslog("esdm-proc");
+	}
+
 	/*
 	 * When --help is specified, first print our own file-system
 	 * specific *help text, then signal fuse_main to show
