@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -24,22 +24,28 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/resource.h>
 #include <unistd.h>
 
-#include "binhexbin.h"
 #include "esdm.h"
 #include "esdm_config.h"
 #include "esdm_rpc_server.h"
 #include "esdm_logger.h"
 #include "ret_checkers.h"
+#include "systemd_support.h"
 
 static unsigned int verbosity = 0;
 static unsigned int foreground = 0;
-/* "/var/run/esdm-rpc-server.pid" */
+/* "/run/esdm-rpc-server.pid" */
 static char *pidfile = NULL;
 static int pidfile_fd = -1;
 static const char *username = NULL;
 
+/*******************************************************************
+ * Forward Declarations
+ *******************************************************************/
+static void daemon_raise_sched_priority(void);
+
 /*******************************************************************
  * General helper functions
  *******************************************************************/
@@ -72,6 +78,10 @@ static void usage(void)
 	fprintf(stderr, "\t\t\t\tretries enabling it\n");
 	fprintf(stderr,
 		"\t   --jent_block_disable\tDisable Jitter RNG block collection\n");
+	fprintf(stderr,
+		"\t-S --syslog\tLog to syslog instead of stdout/stderr\n");
+	fprintf(stderr,
+		"\t-P --raise_sched_priority\tRaise scheduling priority/nice level\n");
 	exit(1);
 }
 
@@ -82,18 +92,21 @@ static void parse_opts(int argc, char *argv[])
 
 	while (1) {
 		int opt_index = 0;
-		static struct option opts[] = { { "verbose", 0, 0, 0 },
-						{ "pid", 1, 0, 0 },
-						{ "help", 0, 0, 0 },
-						{ "version", 0, 0, 0 },
-						{ "username", 0, 0, 0 },
-						{ "foreground", 0, 0, 0 },
-						{ "force_irqes", 0, 0, 0 },
-						{ "force_schedes", 0, 0, 0 },
-						{ "jent_block_disable", 0, 0,
-						  0 },
-						{ 0, 0, 0, 0 } };
-		c = getopt_long(argc, argv, "hvp:u:fis", opts, &opt_index);
+		static struct option opts[] = {
+			{ "verbose", 0, 0, 0 },
+			{ "pid", 1, 0, 0 },
+			{ "help", 0, 0, 0 },
+			{ "version", 0, 0, 0 },
+			{ "username", 0, 0, 0 },
+			{ "foreground", 0, 0, 0 },
+			{ "force_irqes", 0, 0, 0 },
+			{ "force_schedes", 0, 0, 0 },
+			{ "jent_block_disable", 0, 0, 0 },
+			{ "syslog", 0, 0, 0 },
+			{ "raise_sched_priority", 0, 0, 0 },
+			{ 0, 0, 0, 0 }
+		};
+		c = getopt_long(argc, argv, "hvp:u:fisSP", opts, &opt_index);
 		if (-1 == c)
 			break;
 		switch (c) {
@@ -140,6 +153,16 @@ static void parse_opts(int argc, char *argv[])
 				esdm_config_es_jent_async_enabled_set(0);
 				break;
 
+			case 9:
+				/* syslog */
+				esdm_logger_enable_syslog("esdm-server");
+				break;
+
+			case 10:
+				/* raise_sched_priority */
+				daemon_raise_sched_priority();
+				break;
+
 			default:
 				usage();
 			}
@@ -159,6 +182,13 @@ static void parse_opts(int argc, char *argv[])
 		case 'f':
 			foreground = 1;
 			break;
+		case 'S':
+			esdm_logger_enable_syslog("esdm-server");
+			break;
+		case 'P':
+			/* raise_sched_priority */
+			daemon_raise_sched_priority();
+			break;
 
 		case 'i':
 			/* force_irqes */
@@ -184,7 +214,6 @@ static int daemon_init(void)
 	int ret;
 
 	esdm_logger(LOGGER_VERBOSE, LOGGER_C_SERVER, "Starting ESDM server\n");
-	CKINT(esdm_init());
 	CKINT(esdm_rpc_server_init(username));
 
 out:
@@ -197,6 +226,15 @@ static void daemon_release(void)
 	esdm_fini();
 }
 
+static void daemon_raise_sched_priority(void)
+{
+	/* set nice priority */
+	if (setpriority(PRIO_PROCESS, 0, -15) == -1) {
+		esdm_logger(LOGGER_WARN, LOGGER_C_SERVER,
+			    "Raising scheduling priority failed!\n");
+	}
+}
+
 static void dealloc(void)
 {
 	if (pidfile_fd != -1) {
@@ -279,6 +317,15 @@ static void daemonize(void)
 {
 	pid_t pid;
 
+	/* forbid daemonization with socket activation */
+	if (systemd_listen_fds() > 0) {
+		esdm_logger(
+			LOGGER_ERR, LOGGER_C_SERVER,
+			"Do not use systemd socket activation and daemonization together.\n"
+			"Please run esdm-server in foreground mode.\n");
+		exit(1);
+	}
+
 	/* already a daemon */
 	if (getppid() == 1)
 		return;
@@ -344,6 +391,9 @@ int main(int argc, char *argv[])
 
 	CKINT(daemon_init());
 
+	systemd_notify_status("Waiting for subprocesses to terminate");
+	systemd_notify_stopping();
+
 out:
 	daemon_release();
 	dealloc();
