@@ -1,6 +1,6 @@
 /* RPC Client: Connection handler to server
  *
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -19,7 +19,9 @@
  */
 
 #include <errno.h>
+#include <pthread.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
@@ -49,6 +51,20 @@ struct esdm_rpcc_write_buf {
 	esdm_rpc_client_connection_t *rpc_conn;
 };
 
+static void register_fork_handler_unprivileged(void);
+static void register_fork_handler_privileged(void);
+
+static void reset_conn_socket(esdm_rpc_client_connection_t *rpc_conn)
+{
+	if (rpc_conn == NULL) {
+		return;
+	}
+	if (rpc_conn->fd >= 0)
+		close(rpc_conn->fd);
+	rpc_conn->fd = -1;
+	memset(&rpc_conn->last_used, 0, sizeof(rpc_conn->last_used));
+}
+
 static void esdm_fini_proto_service(esdm_rpc_client_connection_t *rpc_conn)
 {
 	ProtobufCService *service;
@@ -57,8 +73,7 @@ static void esdm_fini_proto_service(esdm_rpc_client_connection_t *rpc_conn)
 		return;
 
 	if (rpc_conn->fd >= 0) {
-		close(rpc_conn->fd);
-		rpc_conn->fd = -1;
+		reset_conn_socket(rpc_conn);
 	}
 
 	service = &rpc_conn->service;
@@ -88,8 +103,7 @@ static int esdm_connect_proto_service(esdm_rpc_client_connection_t *rpc_conn)
 	int errsv;
 
 	if (rpc_conn->fd >= 0) {
-		close(rpc_conn->fd);
-		rpc_conn->fd = -1;
+		reset_conn_socket(rpc_conn);
 	}
 
 	/* Does the path exist? */
@@ -111,13 +125,22 @@ static int esdm_connect_proto_service(esdm_rpc_client_connection_t *rpc_conn)
 
 	/* Connect to the Unix domain socket */
 	addr.sun_family = AF_UNIX;
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
 	strncpy(addr.sun_path, socketname, sizeof(addr.sun_path));
-	rpc_conn->fd = socket(addr.sun_family, SOCK_SEQPACKET, 0);
+#pragma GCC diagnostic pop
+
+	rpc_conn->fd =
+		socket(addr.sun_family, SOCK_SEQPACKET | SOCK_CLOEXEC, 0);
 	if (rpc_conn->fd < 0) {
 		errsv = errno;
 
 		esdm_logger(LOGGER_ERR, LOGGER_C_RPC,
 			    "Error creating socket: %s\n", strerror(errsv));
+
+		reset_conn_socket(rpc_conn);
+
 		return -errsv;
 	}
 
@@ -131,6 +154,9 @@ static int esdm_connect_proto_service(esdm_rpc_client_connection_t *rpc_conn)
 		esdm_logger(LOGGER_ERR, LOGGER_C_RPC,
 			    "Error setting timeout on socket: %s\n",
 			    strerror(errsv));
+
+		reset_conn_socket(rpc_conn);
+
 		return -errsv;
 	}
 
@@ -153,10 +179,14 @@ static int esdm_connect_proto_service(esdm_rpc_client_connection_t *rpc_conn)
 	} while (attempts < (ESDM_CLIENT_RECONNECT_ATTEMPTS) &&
 		 (errsv == EAGAIN || errsv == ECONNREFUSED || errsv == EINTR));
 
-	if (errsv) {
+	if (errsv || attempts >= ESDM_CLIENT_RECONNECT_ATTEMPTS) {
 		esdm_logger(LOGGER_ERR, LOGGER_C_RPC,
 			    "Connection attempt using socket %s failed\n",
 			    socketname);
+		reset_conn_socket(rpc_conn);
+	} else {
+		/* only update this time on a successful connection */
+		clock_gettime(CLOCK_MONOTONIC, &rpc_conn->last_used);
 	}
 
 	return -errsv;
@@ -197,6 +227,8 @@ static int esdm_rpc_client_write_data_fd(esdm_rpc_client_connection_t *rpc_conn,
 					LOGGER_DEBUG, LOGGER_C_RPC,
 					"Connection to server needs to be re-established\n");
 
+				reset_conn_socket(rpc_conn);
+
 				int rc = esdm_connect_proto_service(rpc_conn);
 				if (rc)
 					return rc;
@@ -246,6 +278,7 @@ static int esdm_rpc_client_pack(const ProtobufCMessage *message,
 	struct esdm_rpc_write_data_buf tmp = {
 		.dst_written = 0,
 	};
+	size_t buf_alloc_size;
 
 	tmp.base.append = esdm_rpc_append_data;
 
@@ -256,8 +289,9 @@ static int esdm_rpc_client_pack(const ProtobufCMessage *message,
 		return -EFAULT;
 	}
 
-	data_buf_alloc = malloc(ESDM_RPCC_BUF_WRITE_HEADER_SZ + message_length +
-				sizeof(uint64_t) - 1);
+	buf_alloc_size = ESDM_RPCC_BUF_WRITE_HEADER_SZ + message_length +
+			 sizeof(uint64_t) - 1;
+	data_buf_alloc = malloc(buf_alloc_size);
 	CKNULL(data_buf_alloc, -ENOMEM);
 
 	data_buf = ALIGN_PTR_8(data_buf_alloc, sizeof(uint64_t));
@@ -289,12 +323,16 @@ static int esdm_rpc_client_pack(const ProtobufCMessage *message,
 
 out:
 	/*
-	 * Zeroization not needed, as data will go out through unprotected
-	 * channel anyway. If the data is not already protected, there is a
-	 * bigger problem.
+	 * Zeroization only here is not sufficient.
+	 * Make sure to enable zeroize on alloc and free in your Linux kernel
+	 * and clear data from ESDM in your application or patch
+	 * ESDM to include a cryptographic tunnel to your application.
 	 */
-	if (data_buf_alloc)
+	if (data_buf_alloc) {
+		memset_secure(data_buf_alloc, 0, buf_alloc_size);
 		free(data_buf_alloc);
+		data_buf_alloc = NULL;
+	}
 	return ret;
 }
 
@@ -498,6 +536,7 @@ esdm_rpc_client_read_handler(esdm_rpc_client_connection_t *rpc_conn,
 			msg = ERR_PTR(-EFAULT);
 			closure(msg, closure_data);
 		}
+		clock_gettime(CLOCK_MONOTONIC, &rpc_conn->last_used);
 	} else if (interrupted) {
 		ProtobufCMessage *msg;
 
@@ -529,17 +568,32 @@ static void esdm_client_invoke(ProtobufCService *service,
 	const ProtobufCMethodDescriptor *method = desc->methods + method_index;
 	esdm_rpc_client_connection_t *rpc_conn =
 		(esdm_rpc_client_connection_t *)service;
+	static const double half_server_timeout =
+		(double)ESDM_RPC_IDLE_TIMEOUT_USEC / 1E6 / 2.0;
+	struct timespec current_time;
+	double used_before_secs;
 	int ret;
 
 	mutex_w_lock(&rpc_conn->lock);
 
 	do {
+		clock_gettime(CLOCK_MONOTONIC, &current_time);
+		used_before_secs = (double)current_time.tv_sec -
+				   (double)rpc_conn->last_used.tv_sec;
+		used_before_secs += (double)current_time.tv_nsec / 1E9 -
+				    (double)rpc_conn->last_used.tv_nsec / 1E9;
+
 		/*
 		 * Connect to the server if we do not have a connection,
 		 * otherwise reuse the session.
+		 *
+		 * Server keeps the connection open for ESDM_RPC_IDLE_TIMEOUT_USEC, consider it closed a bit earlier.
 		 */
-		if (rpc_conn->fd == -1)
+		if (rpc_conn->fd == -1 ||
+		    used_before_secs >= half_server_timeout) {
+			reset_conn_socket(rpc_conn);
 			CKINT(esdm_connect_proto_service(rpc_conn));
+		}
 
 		/* Pack the protobuf-c data and send it over the wire */
 		CKINT_LOG(esdm_rpc_client_pack(input, method_index, rpc_conn),
@@ -562,8 +616,7 @@ static void esdm_client_destroy(ProtobufCService *service)
 
 	mutex_w_lock(&rpc_conn->lock);
 	if (rpc_conn->fd >= 0) {
-		close(rpc_conn->fd);
-		rpc_conn->fd = -1;
+		reset_conn_socket(rpc_conn);
 	}
 	mutex_w_unlock(&rpc_conn->lock);
 }
@@ -589,6 +642,7 @@ static int esdm_init_proto_service(const ProtobufCServiceDescriptor *descriptor,
 
 	mutex_w_init(&rpc_conn->ref_cnt, 0, 1);
 	rpc_conn->fd = -1;
+	reset_conn_socket(rpc_conn);
 	mutex_w_init(&rpc_conn->lock, 0, 1);
 	atomic_set(&rpc_conn->state, esdm_rpcc_initialized);
 
@@ -599,7 +653,7 @@ out:
 /******************************************************************************
  * General service handlers
  ******************************************************************************/
-static uint32_t esdm_rpcc_max_nodes = 0xffffffff;
+static uint32_t esdm_rpcc_max_nodes = UINT32_MAX;
 
 DSO_PUBLIC
 int esdm_rpcc_set_max_online_nodes(uint32_t nodes)
@@ -761,19 +815,65 @@ static int esdm_rpcc_get_service(esdm_rpc_client_connection_t *rpc_conn_array,
 	esdm_rpc_client_connection_t *rpc_conn_p;
 	/* Protection against client programming errors */
 	uint32_t node = min_uint32(esdm_rpcc_curr_node(), num_conn);
+	bool found_unused_conn = false;
+	struct timespec abstime;
+	int lock_res;
 	int ret = 0;
+	uint32_t i;
+	int32_t j;
 
 	CKNULL(rpc_conn_array, -EFAULT);
 	CKNULL(ret_rpc_conn, -EFAULT);
 
-	rpc_conn_p = rpc_conn_array + node;
-
 	/*
-	 * Wait until the previous call completed - each connection handle is
-	 * has only one caller at one given time. Lock the ref_cnt if we
-	 * obtained the connection handle.
+	 * Always using a fixed connection based on the current
+	 * core slows the client down, as under load a thread waiting
+	 * on a reply from ESDM is paused and probably another thread
+	 * also communicating with ESDM scheduled on the same node
+	 * waiting for the RPC connection to become available again
+	 * with no gain.
+	 *
+	 * Try to optimistically find a free connection slot first.
 	 */
-	mutex_w_lock(&rpc_conn_p->ref_cnt);
+	for (i = 0; i < num_conn; ++i) {
+		rpc_conn_p = rpc_conn_array + (i + node) % num_conn;
+		if (mutex_w_trylock(&rpc_conn_p->ref_cnt)) {
+			found_unused_conn = true;
+			break;
+		}
+	}
+
+	/* as escalation step try timed locking */
+	if (!found_unused_conn && num_conn > 0) {
+		/* try in reverse order of probing above for less lock contention */
+		for (j = (int32_t)num_conn - 1; j >= 0; --j) {
+			/* every core probes at another offset */
+			rpc_conn_p = rpc_conn_array +
+				     ((uint32_t)j + node) % num_conn;
+
+			clock_gettime(CLOCK_MONOTONIC, &abstime);
+			// 5ms
+			abstime.tv_nsec += 5000000;
+			lock_res = mutex_w_timedlock(&rpc_conn_p->ref_cnt,
+						     &abstime);
+
+			if (lock_res == 0) {
+				found_unused_conn = true;
+				break;
+			}
+		}
+	}
+
+	if (!found_unused_conn) {
+		rpc_conn_p = rpc_conn_array + node % num_conn;
+
+		/*
+		* Wait until the previous call completed - each connection handle is
+		* has only one caller at one given time. Lock the ref_cnt if we
+		* obtained the connection handle.
+		*/
+		mutex_w_lock(&rpc_conn_p->ref_cnt);
+	}
 
 	if (atomic_read(&rpc_conn_p->state) != esdm_rpcc_initialized) {
 		mutex_w_unlock(&rpc_conn_p->ref_cnt);
@@ -822,6 +922,8 @@ void esdm_rpcc_put_unpriv_service(esdm_rpc_client_connection_t *rpc_conn)
 DSO_PUBLIC
 int esdm_rpcc_init_unpriv_service(esdm_rpcc_interrupt_func_t interrupt_func)
 {
+	register_fork_handler_unprivileged();
+
 	return esdm_rpcc_init_service(&unpriv_access__descriptor,
 				      ESDM_RPC_UNPRIV_SOCKET, interrupt_func,
 				      &unpriv_rpc_conn, &unpriv_rpc_conn_num);
@@ -856,6 +958,8 @@ void esdm_rpcc_put_priv_service(esdm_rpc_client_connection_t *rpc_conn)
 DSO_PUBLIC
 int esdm_rpcc_init_priv_service(esdm_rpcc_interrupt_func_t interrupt_func)
 {
+	register_fork_handler_privileged();
+
 	return esdm_rpcc_init_service(&priv_access__descriptor,
 				      ESDM_RPC_PRIV_SOCKET, interrupt_func,
 				      &priv_rpc_conn, &priv_rpc_conn_num);
@@ -866,3 +970,57 @@ void esdm_rpcc_fini_priv_service(void)
 {
 	esdm_rpcc_fini_service(&priv_rpc_conn, &priv_rpc_conn_num);
 }
+
+/******************************************************************************
+ * Fork Handling
+ ******************************************************************************/
+static pid_t owner_pid_unprivileged = -1;
+static pid_t owner_pid_privileged = -1;
+
+/* does nothing, if no connections were allocated and nums are 0 */
+static void cleanup_after_fork_unprivileged()
+{
+	uint32_t i;
+
+	/* close all unprivileged sockets */
+	for (i = 0; i < unpriv_rpc_conn_num; ++i) {
+		reset_conn_socket(&unpriv_rpc_conn[i]);
+	}
+}
+
+/* does nothing, if no connections were allocated and nums are 0 */
+static void cleanup_after_fork_privileged()
+{
+	uint32_t i;
+
+	/* close all privileged sockets */
+	for (i = 0; i < priv_rpc_conn_num; ++i) {
+		reset_conn_socket(&priv_rpc_conn[i]);
+	}
+}
+
+/* need different handlers in order to not interfere with the other case, when opened
+ * in two threads at different times */
+static void register_fork_handler_unprivileged(void)
+{
+	/* also works in the initial call */
+	if (getpid() != owner_pid_unprivileged) {
+		owner_pid_unprivileged = getpid();
+		cleanup_after_fork_unprivileged();
+	}
+
+	pthread_atfork(NULL, NULL, &register_fork_handler_unprivileged);
+}
+
+/* need different handlers in order to not interfere with the other case, when opened
+ * in two threads at different times */
+static void register_fork_handler_privileged(void)
+{
+	/* also works in the initial call */
+	if (getpid() != owner_pid_privileged) {
+		owner_pid_privileged = getpid();
+		cleanup_after_fork_privileged();
+	}
+
+	pthread_atfork(NULL, NULL, &register_fork_handler_privileged);
+}
