@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -47,7 +47,11 @@ static void esdm_rpcc_get_seed_cb(const GetSeedResponse *response,
 
 	buffer->ret = response->ret;
 	buffer->buflen = min_size(response->randval.len, buffer->buflen);
-	memcpy(buffer->buf, response->randval.data, buffer->buflen);
+
+	/* only copy if data present */
+	if (response->ret > 0 && response->randval.data != NULL) {
+		memcpy(buffer->buf, response->randval.data, buffer->buflen);
+	}
 
 	/* Zeroization of response is handled in esdm_rpc_client_read_handler */
 }
