@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -25,6 +25,7 @@
 #include "esdm_rpc_server.h"
 #include "esdm_rpc_service.h"
 #include "helper.h"
+#include "math_helper.h"
 #include "esdm_logger.h"
 #include "memset_secure.h"
 #include "threading_support.h"
@@ -42,8 +43,8 @@ void esdm_rpc_get_seed(UnprivAccess_Service *service,
 		response.ret = -(int32_t)sizeof(rndval);
 		closure(&response, closure_data);
 	} else {
-		/* TODO: make 280 dependent on output size */
-		memset(rndval, 0, 280);
+		memset(rndval, 0, min_size(sizeof(rndval), request->len));
+
 		response.ret =
 			esdm_get_seed(rndval, request->len,
 				      request->flags | ESDM_GET_SEED_NONBLOCK);
