@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -21,6 +21,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <netinet/in.h>
+#include <poll.h>
 #include <protobuf-c/protobuf-c.h>
 #include <semaphore.h>
 #include <signal.h>
@@ -28,7 +29,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/shm.h>
-#include <sys/select.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
@@ -44,7 +44,6 @@
 #include "esdm_rpc_protocol.h"
 #include "esdm_rpc_protocol_helper.h"
 #include "esdm_rpc_server.h"
-#include "esdm_rpc_server_linux.h"
 #include "esdm_rpc_service.h"
 #include "helper.h"
 #include "linux_support.h"
@@ -53,6 +52,7 @@
 #include "privileges.h"
 #include "ret_checkers.h"
 #include "queue.h"
+#include "systemd_support.h"
 #include "threading_support.h"
 
 struct esdm_rpcs {
@@ -553,7 +553,8 @@ static int esdm_rpcs_workerloop(struct esdm_rpcs *proto)
 	 * to pay for not using malloc and a thread-local storage
 	 * buffer.
 	 */
-	struct timeval tv = { .tv_sec = 2, .tv_usec = 0 };
+	struct timeval tv = { .tv_sec = ESDM_RPC_IDLE_TIMEOUT_USEC / 1000000,
+			      .tv_usec = ESDM_RPC_IDLE_TIMEOUT_USEC % 1000000 };
 	struct esdm_rpcs_connection *rpc_conn = NULL;
 	struct sockaddr addr;
 	socklen_t addr_len = sizeof(addr);
@@ -609,18 +610,18 @@ static int esdm_rpcs_workerloop(struct esdm_rpcs *proto)
 		 */
 #ifndef ESDM_WORKERLOOP_TERM_ON_SIGNAL
 		for (;;) {
-			struct timeval timeout = { .tv_sec = 1, .tv_usec = 0 };
-			fd_set fds;
-			int sret;
+			struct timespec timeout = { .tv_sec = 1, .tv_nsec = 0 };
+			struct pollfd pfd = {
+				.fd = proto->server_listening_fd,
+				.events = POLL_IN,
+				.revents = 0
+			};
+			int pret;
 
-			FD_ZERO(&fds);
-			FD_SET(proto->server_listening_fd, &fds);
-
-			sret = select(proto->server_listening_fd + 1, &fds,
-				      NULL, NULL, &timeout);
+			pret = ppoll(&pfd, 1,&timeout, NULL);
 
 			/* error */
-			if (sret == -1 && errno != EINTR) {
+			if (pret == -1 && errno != EINTR) {
 				esdm_logger(LOGGER_ERR, LOGGER_C_ANY,
 					    "Select returned with error %s\n",
 					    strerror(errno));
@@ -628,7 +629,7 @@ static int esdm_rpcs_workerloop(struct esdm_rpcs *proto)
 			}
 
 			/* activity */
-			if (sret > 0)
+			if (pret > 0)
 				break;
 
 			/* timeout - yet server shall exit */
@@ -720,8 +721,11 @@ static int esdm_rpcs_start(const char *unix_socket, uint16_t tcp_port,
 		protocol_family = PF_UNIX;
 		memset(&addr_un, 0, sizeof(addr_un));
 		addr_un.sun_family = AF_UNIX;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
 		strncpy(addr_un.sun_path, unix_socket,
 			sizeof(addr_un.sun_path));
+#pragma GCC diagnostic pop
 		address_len = sizeof(addr_un);
 		address = (struct sockaddr *)(&addr_un);
 
@@ -761,7 +765,7 @@ static int esdm_rpcs_start(const char *unix_socket, uint16_t tcp_port,
 		return -errsv;
 	}
 
-	if (listen(fd, 255) < 0) {
+	if (listen(fd, 5 * THREADING_MAX_THREADS) < 0) {
 		errsv = -errno;
 		esdm_logger(LOGGER_ERR, LOGGER_C_RPC,
 			    "RPC Server: cannot listen on socket: %s\n",
@@ -776,6 +780,59 @@ static int esdm_rpcs_start(const char *unix_socket, uint16_t tcp_port,
 	return 0;
 }
 
+/* Use the socket that we want to use for receiving data. */
+static int esdm_rpcs_start_systemd(const char *socket_name,
+				   ProtobufCService *service,
+				   struct esdm_rpcs *proto)
+{
+#ifdef ESDM_SYSTEMD_SUPPORT
+	int fd = -1;
+	int type;
+	socklen_t length = sizeof(type);
+
+	if (systemd_listen_fds() > 0) {
+		fd = systemd_listen_fd_for_name(socket_name);
+	}
+	if (fd >= 0) {
+		esdm_logger(LOGGER_DEBUG, LOGGER_C_SERVER,
+			    "use systemd provided socket %s = %i\n",
+			    socket_name, fd);
+
+		/* double check, that we are responsible for this socket/fd */
+		assert(systemd_listen_pid() == getppid());
+
+		if (getsockopt(fd, SOL_SOCKET, SO_TYPE, &type, &length) == -1) {
+			esdm_logger(
+				LOGGER_ERR, LOGGER_C_SERVER,
+				"unable to fetch socket type of systemd provided socket");
+			close(fd);
+			return -1;
+		}
+
+		if (type != SOCK_SEQPACKET) {
+			esdm_logger(
+				LOGGER_ERR, LOGGER_C_SERVER,
+				"systemd provided socket is not of type SOCK_SEQPACKET");
+			close(fd);
+			return -1;
+		}
+
+		proto->server_listening_fd = fd;
+		proto->service = service;
+		return 0;
+	}
+
+	esdm_logger(LOGGER_WARN, LOGGER_C_SERVER,
+		    "unable to find systemd provided socket %s\n", socket_name);
+	return -1;
+#else
+	(void)socket_name;
+	(void)service;
+	(void)proto;
+	return 0;
+#endif
+}
+
 /* Terminating the RPC server. */
 static void eesdm_rpcs_stop(struct esdm_rpcs *proto)
 {
@@ -801,8 +858,13 @@ static int esdm_rpcs_unpriv_init(void *args)
 	unpriv_proto.server_listening_fd = -1;
 
 	/* Create server handler for privileged interface in main thread */
-	CKINT(esdm_rpcs_start(ESDM_RPC_UNPRIV_SOCKET, 0, unpriv_service,
-			      &unpriv_proto));
+	if (systemd_listen_fds() > 0) {
+		CKINT(esdm_rpcs_start_systemd("ESDM_RPC_UNPRIV_SOCKET",
+					      unpriv_service, &unpriv_proto));
+	} else {
+		CKINT(esdm_rpcs_start(ESDM_RPC_UNPRIV_SOCKET, 0, unpriv_service,
+				      &unpriv_proto));
+	}
 
 	/* Make unprivileged socket available for all users */
 	if (chmod(ESDM_RPC_UNPRIV_SOCKET,
@@ -859,8 +921,13 @@ static int esdm_rpcs_interfaces_init(const char *username)
 	memset(&priv_proto, 0, sizeof(priv_proto));
 
 	/* Create server handler for privileged interface in main thread */
-	CKINT(esdm_rpcs_start(ESDM_RPC_PRIV_SOCKET, 0, priv_service,
-			      &priv_proto));
+	if (systemd_listen_fds() > 0) {
+		CKINT(esdm_rpcs_start_systemd("ESDM_RPC_PRIV_SOCKET",
+					      priv_service, &priv_proto));
+	} else {
+		CKINT(esdm_rpcs_start(ESDM_RPC_PRIV_SOCKET, 0, priv_service,
+				      &priv_proto));
+	}
 
 	/* Make privileged socket available for root only */
 	if (chmod(ESDM_RPC_PRIV_SOCKET, S_IRUSR | S_IWUSR) == -1) {
@@ -894,6 +961,13 @@ static int esdm_rpcs_interfaces_init(const char *username)
 		    "Privileged server thread for %s available\n",
 		    ESDM_RPC_PRIV_SOCKET);
 
+	systemd_notify_status("ESDM ready, all sockets allocated");
+	systemd_notify_ready();
+	systemd_notify_status("Running");
+
+	/* drop notify privileges to main process only again */
+	systemd_notify_access("main");
+
 	/* Server handing privileged interface in current thread */
 	CKINT(esdm_rpcs_workerloop(&priv_proto));
 
@@ -1009,17 +1083,20 @@ static int esdm_rpc_server_es_monitor(void __unused *unused)
 
 int esdm_rpc_server_init(const char *username)
 {
-	pid_t pid;
 	int ret = 0;
+	pid_t pid;
 
 	/* Enter PID name space */
 	CKINT(linux_isolate_namespace_prefork());
 
-	/* Initialize test pertubation support */
-	CKINT(esdm_test_shm_status_init());
+	/*
+	 * allow all child processes to notify systemd of successfull launch.
+	 * drop this again early in child after sockets are ready.
+	 */
+	systemd_notify_status("Waiting for RPC process to notify readiness");
+	systemd_notify_access("all");
 
-	/* One thread group */
-	CKINT(thread_init(1));
+	esdm_logger(LOGGER_WARN, LOGGER_C_SERVER, "Forking RPC Server\n");
 
 	pid = fork();
 	if (pid < 0) {
@@ -1029,6 +1106,15 @@ int esdm_rpc_server_init(const char *username)
 	} else if (pid == 0) {
 		pthread_setname_np(pthread_self(), "ESDM master");
 
+		/* Main ESDM Init DRNG state, ES', ... */
+		CKINT(esdm_init());
+
+		/* Initialize test pertubation support */
+		CKINT(esdm_test_shm_status_init());
+
+		/* One thread group */
+		CKINT(thread_init(1));
+
 		/* Create thread for entropy source monitor */
 		if (thread_start(esdm_rpc_server_es_monitor, NULL,
 				 ESDM_THREAD_ES_MONITOR, NULL)) {
@@ -1041,9 +1127,21 @@ int esdm_rpc_server_init(const char *username)
 				  (atomic_read(&esdm_rpc_init_state) ==
 				   esdm_rpcs_state_priv_init_complete));
 
-		/* Fork the server process */
+		esdm_logger(LOGGER_WARN, LOGGER_C_RPC, "RPC server started\n");
+
+		/* start the RPC server threads */
 		esdm_rpcs_interfaces_init(username);
 	} else {
+		/* cleanup systemd sockets */
+		if (systemd_listen_fds() > 0) {
+			for (int fd = SYSTEMD_LISTEN_FDS_START;
+			     fd <
+			     SYSTEMD_LISTEN_FDS_START + systemd_listen_fds();
+			     ++fd) {
+				close(fd);
+			}
+		}
+
 		/*
 		 * This is the cleanup process. It simply waits for the server
 		 * to exit to clean up its resources. This is needed because
@@ -1063,17 +1161,15 @@ int esdm_rpc_server_init(const char *username)
 		server_pid = pid;
 		esdm_rpcs_cleanup_signals(esdm_rpcs_cleanup_term);
 
-		/* Cannot do anything with the return code, ignoring. */
-		esdm_rpcs_linux_init_feeder();
-
 		/* Now wait for the server to finish. */
 		waitpid(pid, NULL, 0);
 		server_pid = -1;
 
 		esdm_rpcs_cleanup_signals(SIG_DFL);
 
-		/* Clean up all resources */
-		esdm_rpcs_cleanup();
+		/* Clean up all resources only, when no systemd provided sockets are used */
+		if (systemd_listen_fds() == 0)
+			esdm_rpcs_cleanup();
 	}
 
 out:
