@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 - 2024, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2022 - 2025, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -52,9 +52,11 @@ extern "C" {
 
 #else /* ESDM_TESTMODE */
 
-#define ESDM_RPC_UNPRIV_SOCKET "/var/run/esdm-rpc-unpriv.socket"
+#define ESDM_RPC_UNPRIV_SOCKET                                                 \
+	ESDM_SERVER_RPC_BASE_PATH_UNPRIVILEGED "/esdm-rpc-unpriv.socket"
 
-#define ESDM_RPC_PRIV_SOCKET "/var/run/esdm-rpc-priv.socket"
+#define ESDM_RPC_PRIV_SOCKET                                                   \
+	ESDM_SERVER_RPC_BASE_PATH_PRIVILEGED "/esdm-rpc-priv.socket"
 
 #define ESDM_SHM_NAME "/"
 #define ESDM_SHM_STATUS 0x6d647365
@@ -125,8 +127,7 @@ void esdm_rpc_get_random_bytes_full(UnprivAccess_Service *service,
 void esdm_rpc_get_random_bytes_full_timeout(
 	UnprivAccess_Service *service,
 	const GetRandomBytesFullTimeoutRequest *request,
-	GetRandomBytesFullTimeoutResponse_Closure closure,
-	void *closure_data);
+	GetRandomBytesFullTimeoutResponse_Closure closure, void *closure_data);
 
 void esdm_rpc_get_random_bytes_min(UnprivAccess_Service *service,
 				   const GetRandomBytesMinRequest *request,
